/*
 * $Id: DbSource.java 5211 2020-11-04 20:28:21Z carsten $
 *
 * Copyright 2018 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.annotations;

import static java.lang.annotation.ElementType.TYPE;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Documented;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;

import de.cm.osm2po.postp.DbExtensionBuilder;
import de.cm.osm2po.primitives.Var;

/**
 * Annotiert eine Klasse als Stellvertreter einer Tabelle.
 * <br><br><b>Wichtig:</b><br>
 * Die Klasse sollte zusaetzlich {@link Var} implementieren,
 * damit sie sinnvoll verwendet werden kann.
 * @author (c) 2018 - Carsten Moeller, info@osm2po.de
 * @see DbColumn
 * @see DbExtensionBuilder
 */
@Retention(RUNTIME)
@Target(TYPE)
@Documented
public @interface DbSource {
    /** @return Datenquelle als SQL */
    String sql();
    /** @return Abweichender Name der Indexspalte*/
    String index() default "id";
    /** @return Eindeutiger alias fuer spaeteren Zugriff */
    String alias() default "";
}
