/*
 * $Id: HandlerMethod.java 5211 2020-11-04 20:28:21Z carsten $
 *
 * Copyright 2017 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.annotations;

import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Documented;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;

import de.cm.osm2po.service.Osm2poService;
import de.cm.osm2po.service.ServiceHandlerProxy;

/**
 * Annotiert Methoden fuer die Verwendung als Handler fuer
 * z.B. den {@link Osm2poService}.
 * @author (c) 2018 - Carsten Moeller, info@osm2po.de
 * @see ServiceHandlerProxy
 */
@Retention(RUNTIME)
@Target(METHOD)
@Documented
public @interface HandlerMethod {
    /** @return Abweichender Name fuer cmd-Parameter, sonst normalerweise Methoden-Name **/
    String cmd() default "";
    /** @return Optionale URL-Pfad-Mappings als CSV fuer diesen cmd **/
    String[] mapping() default {};
    /** @return Marker fuer kostenintensive Berechnungen */
    boolean heavy() default false;
    /** @return Methode nicht als Handler registrieren */
    boolean ignore() default false;
    /** @return HTTP-Methode, default ist GET */
    String method() default "GET";
}
