/*
 * $Id: JoinFilter.java 5211 2020-11-04 20:28:21Z carsten $
 *
 * Copyright 2011/2020 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.converter.joiner;

import de.cm.osm2po.Config;
import de.cm.osm2po.converter.Joiner;
import de.cm.osm2po.converter.tiler.Tile;
import de.cm.osm2po.model.Way;

/**
 * Schnittstelle, um Wege vor dem endgueltigen Schreiben
 * im {@link Joiner} benutzerdefiniert zu validieren und zu modifizieren.
 * Liefert {@link #validate(Way)} <code>false</code>, so wird dieser verworfen!
 * <br><b>Wichtig:</b><br>
 * Dies ist eine sehr kritische Offenlegung der Eingeweide
 * von osm2po.<br>Unqualifizierte Modifikationen des {@link Way}s
 * koennen zu undefinierten Ergebnissen fuehren.
 * @author (c) 2012 - Carsten Moeller - info@osm2po
 * @see DefaultJoinFilter
 * @see FlowJoinFilter
 * @see #open(Config, String)
 * @see #close()
 * @see #validate(Way)
 * @see #tiledInto(int, boolean)
 */
public interface JoinFilter {

    public final static String PROP_PREFIX = "joinFilter.";

    /**
     * Wird einmalig nach der Instanziierung aufgerufen.
     * Dies ist faktisch der Quasi-Konstruktor.
     * @param config {@link Config} Konfiguration.
     * @param prefix String Property-Prefix ohne Punkt-Trenner NULLABLE.
     * @see FlowJoinFilter
     */
    void open(Config config, String prefix);

    /**
     * Schliesst und bietet die Moeglichkeit, resourcen
     * aufzuraeumen.
     */
    void close();

    /**
     * Kritischer Callback, der einen Verweis auf den gerade
     * zusammengestellten Weg uebergibt und somit alle
     * Tueren fuer letzte Manipulationen vorm endgueltigen
     * Rausschreiben oeffnet.
     * @param way {@link Way}
     * @return boolean Valide ja/nein
     */
    boolean validate(Way way);

    /**
     * Wird immer nach {@link #validate(Way)} aufgerufen und informiert darueber,
     * welches {@link Tile} der {@link Joiner} dem {@link Way} zugeordnet hat,
     * bzw. ob der Weg ueberhaupt geschrieben wurde.
     * @param tileId int ID des Tiles oder -1 wenn Shared, bzw. -2 wenn orphaned-shared.
     * @param written true: {@link Way} wurde vom {@link Joiner} geschrieben.
     */
    void tiledInto(int tileId, boolean written);

}
