/*
 * $Id: OsmParserHandler.java 4606 2018-01-11 21:50:57Z carsten $
 *
 * Copyright 2011 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.converter.parser;

import de.cm.osm2po.Config;
import de.cm.osm2po.Main;
import de.cm.osm2po.converter.Tiler;
import de.cm.osm2po.model.OsmNode;
import de.cm.osm2po.model.OsmRelation;
import de.cm.osm2po.model.OsmWay;

/**
 * Implementationen dieses Interfaces werden von einem
 * {@link OsmParser} waehrend der Parsing-Phase ueber
 * die Eigenschaften von Nodes, Ways und Relations informiert.
 *
 * @author (c) 2012 - Carsten Moeller - info@osm2po
 * @see OsmXmlParser
 * @see Tiler
 */
public interface OsmParserHandler {
    /**
     * Quasi-Konstruktor, der normalerweise von {@link Main}
     * aufgerufen wird.
     * <br><br><strong>Wichtig:</strong><br>
     * Sollte nicht vom {@link OsmParser} gerufen werden.
     * @param config {@link Config}
     */
    void open(Config config);

    /**
     * Quasi-Destruktor, der normalerweise von {@link Main}
     * aufgerufen wird.
     * <br><br><strong>Wichtig:</strong><br>
     * Sollte nicht vom {@link OsmParser} gerufen werden.
     */
    void close();

    void onNode(long id, double lat, double lon);
    void onNodeAction(String action);
    void onNodeTag(String key, String value);
    void onNodeObj(String key, Object value);
    void onNodeComplete();

    /**
     * Schliesst alle Resourcen, die fuer das Parsen
     * von {@link OsmNode}s benoetigt wurden.
     */
    void closeNodeProcessing();

    void onWay(long id);
    void onWayNdRef(long ndRefId);
    void onWayTag(String key, String value);
    void onWayObj(String key, Object value);
    void onWayComplete();

    /**
     * Schliesst alle Resourcen, die fuer das Parsen
     * von {@link OsmWay}s benoetigt wurden.
     */
    void closeWayProcessing();

    void onRelation(long id);
    void onRelationMember(String type, long ref, String role);
    void onRelationTag(String key, String value);
    void onRelationComplete();

    /**
     * Schliesst alle Resourcen, die fuer das Parsen
     * von {@link OsmRelation}s benoetigt wurden.
     */
    void closeRelationProcessing();
}