/*
 * $Id: DefaultNodeTagResolver.java 5784 2023-04-09 18:19:13Z carsten $
 *
 * Copyright 2010 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.converter.tiler;

import de.cm.osm2po.Config;
import de.cm.osm2po.primitives.Var;

/**
 * Registriert eine Menge von OSM-Tags eines Knotens
 * und verdichtet diese auf die fuer ein Standard-Routing
 * noetigen Informationen.<br>
 * Es werden hier lediglich {@link #isBarrier()} und
 * {@link #isLevelCrossing()} aufgrund von
 * <code>railway=level_crossing</code> und
 * <code>barrier=bollard</code> ueberschrieben.
 * Alle anderen Schnittstellen liefern Defaults.
 * @author (c) 2012 - Carsten Moeller - info@osm2po
 * @see SimpleNodeTagResolver
 * @see TagResolver
 */
public class DefaultNodeTagResolver implements NodeTagResolver {

    private boolean levelCrossing; // Andreaskreuz oder Bahnuebergang
    private boolean barrier;
    private Var<?> custom;

    @Override
    public void onTags(long entityId) {
        this.levelCrossing = false;
        this.barrier = false;
        this.custom = null;
    }

    @Override
    public void onTag(String key, String value) {
        if ("railway".equals(key)) {
            if ("level_crossing".equals(value)) {
                this.levelCrossing = true;
//            } else if ("halt".equals(value)) { // z.B. ShuttleTrain-Ramp
//                System.out.println("halt");
            }
        } else if ("barrier".equals(key) && "bollard".equals(value)) {
            this.barrier = true;
        }
    }

    @Override
    public void onObj(String key, Object value) {
        if (TAG_CUSTOM.equals(key) && value instanceof Var<?>) {
            this.custom = (Var<?>) value;
        }
    }

    @Override public boolean isLevelCrossing() {return this.levelCrossing;}
    @Override public boolean isBarrier() {return this.barrier;}
    @Override public Var<?> getCustom() {return this.custom;}

    @Override public void open(Config config) {}
    @Override public void close() {}
    @Override public void onTagsComplete() {}
    @Override public byte getClazz() {return CLAZZ_NULL;}
    @Override public int getFlags() {return FLAGS_NULL;}
    @Override public boolean isValid() {return true;}
    @Override public String getName() {return null;}
    @Override public void onCoordinate(double lat, double lon) {}
    @Override public void onAction(String action) {}

}
