/*
 * $Id: Poi2poWriterTagResolver.java 4561 2017-11-15 15:14:00Z carsten $
 *
 * Copyright 2017 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.converter.tiler;

import static de.cm.osm2po.misc.Utils.DF;
import static de.cm.osm2po.primitives.VarTypeDesk.typeIdOf;

import java.io.File;

import de.cm.osm2po.Config;
import de.cm.osm2po.logging.Log;
import de.cm.osm2po.model.LatLon;
import de.cm.osm2po.model.OsmNode;
import de.cm.osm2po.postp.PoiExtensionBuilder2po;
import de.cm.osm2po.primitives.OutStream;
import de.cm.osm2po.primitives.OutStreamDisk;
import de.cm.osm2po.primitives.VarString;

/**
 * Erbt von {@link SimpleNodeTagResolver} und ist zusaetzlich in der Lage,
 * Knoten mit {@link #getClazz()} != 0 in eine Datei im osm2po-Format
 * zu schreiben.<br>
 * Der Ausgabe-Dateiname ist &lt;prefix&gt;_pois.2po.
 * @author (c) 2017 - Carsten Moeller - info@osm2po
 * @see PoiExtensionBuilder2po
 */
public class Poi2poWriterTagResolver extends SimpleNodeTagResolver {

    private OsmNode nodeHolder;
    private OutStream os;
    private long nWritten;
    private Log log;

    @Override
    public void open(Config config) {
        super.open(config);
        this.log = config.getLog();
        this.nWritten = 0;
        this.nodeHolder = new OsmNode();

        File poisFile = new File(config.getWorkDir(),
                config.getPrefix() + "_pois.2po");
        this.os = new OutStreamDisk(poisFile);
        this.os.writeByte(typeIdOf(OsmNode.class));
        this.log.info(this.getClass().getSimpleName() + " configured");
    }

    @Override
    public void close() {
        super.close();
        this.os.close();
        this.log.info(DF(this.nWritten) + " POIs extracted");
    }

    @Override
    public void onTags(long entityId) {
        super.onTags(entityId);
        this.nodeHolder.clear();
        this.nodeHolder.setId(entityId);
    }

    @Override
    public void onCoordinate(double lat, double lon) {
        super.onCoordinate(lat, lon);
        this.nodeHolder.setCoord(LatLon.toCoord(lat, lon));
    }

    @Override
    public void onTagsComplete() {
        super.onTagsComplete();

        byte clazz = super.getClazz();
        if (clazz != CLAZZ_NULL) {
            this.nodeHolder.setName(new VarString(super.getName()));
            this.nodeHolder.setClazz(clazz);
            this.nodeHolder.writeToStream(this.os);
            this.nWritten++;
        }
    }

}
