/*
 *
 * Copyright 2010 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.converter.tiler;

/**
 * Interface fuer Callback-Klassen, die eine Sammlung von
 * OSM-Relation-Tags waehrend der Parsing-Phase auf eindeutige
 * Aussagen verdichten.
 *
 * @author (c) 2012 - Carsten Moeller - info@osm2po
 */
public interface RelationTagResolver extends TagResolver {
    // Nativ (nicht user-defined) ist clazz immer negativ, da Bit 8 gesetzt.
    // Die naechsten 3 HighBits werden fuer die Gruppierung, z.B. Restriktion,
    // Route oder Boundary verwendet. Die restlichen 4 Bits bilden die Auspraegung 0-15.
    // Fuer Polygon-Klassen sollte Bit 7 gesetzt sein!
    // Fuer Restriktionen muessen no_turns immer ungerade sein!
    // Boundaries sind so geschluesselt, dass normale Admin-Levels
    // positive Konstanten sind, Ausreisser hingegen eher negative.
    // Polygone koennen sehr komplex getaggt sein, daher muessen
    // diese im Resolver explizit behandelt werden.

    public final static byte CLAZZ_MASK = (byte) 0xF0; // (1111xxxx)

    public final static byte CLAZZ_RESTRICTION = (byte) 0x80; //-128 (1000xxxx)
    public final static byte CLAZZ_ROUTE = (byte) 0x90; //-112 (1001xxxx)
    public final static byte CLAZZ_BOUNDARY = (byte) 0xC0; //-64 (1100xxxx)
    public final static byte CLAZZ_POLYGON = (byte) 0xD0; //-48 (1101xxxx)

    public final static byte CLAZZ_RESTRICTION_NO_LEFT_TURN = CLAZZ_RESTRICTION | 0x1; // -127
    public final static byte CLAZZ_RESTRICTION_ONLY_LEFT_TURN = CLAZZ_RESTRICTION | 0x2; // -126
    public final static byte CLAZZ_RESTRICTION_NO_RIGHT_TURN = CLAZZ_RESTRICTION | 0x3; // -125
    public final static byte CLAZZ_RESTRICTION_ONLY_RIGHT_TURN = CLAZZ_RESTRICTION | 0x4; // -124
    public final static byte CLAZZ_RESTRICTION_NO_STRAIGHT_ON = CLAZZ_RESTRICTION | 0x5; // -123
    public final static byte CLAZZ_RESTRICTION_ONLY_STRAIGHT_ON = CLAZZ_RESTRICTION | 0x6; // -122
    public final static byte CLAZZ_RESTRICTION_NO_U_TURN = CLAZZ_RESTRICTION | 0x7; // -121
    public final static byte CLAZZ_RESTRICTION_NO_ENTRY = CLAZZ_RESTRICTION | 0x9; // -119
    public final static byte CLAZZ_RESTRICTION_NO_EXIT = CLAZZ_RESTRICTION | 0xB; // -117

    public final static byte CLAZZ_ROUTE_ROAD = CLAZZ_ROUTE | 0x1; // -111
    public final static byte CLAZZ_ROUTE_HIKING = CLAZZ_ROUTE | 0x2; // -110
    public final static byte CLAZZ_ROUTE_BICYCLE  = CLAZZ_ROUTE | 0x3; // -109
    public final static byte CLAZZ_ROUTE_MTB = CLAZZ_ROUTE | 0x4; // -108
    public final static byte CLAZZ_ROUTE_SKI = CLAZZ_ROUTE | 0x5; // -107
    public final static byte CLAZZ_ROUTE_HORSE = CLAZZ_ROUTE | 0x6; // -106
    public final static byte CLAZZ_ROUTE_BUS = CLAZZ_ROUTE | 0x7; // -105
    public final static byte CLAZZ_ROUTE_TRAIN = CLAZZ_ROUTE | 0x8; // -104
    public final static byte CLAZZ_ROUTE_RAILWAY = CLAZZ_ROUTE | 0x9; // -103
    public final static byte CLAZZ_ROUTE_SUBWAY = CLAZZ_ROUTE | 0xA; // -102
    public final static byte CLAZZ_ROUTE_TRAM = CLAZZ_ROUTE | 0xB; // -101
    public final static byte CLAZZ_ROUTE_FERRY = CLAZZ_ROUTE | 0xC; // -100
    public final static byte CLAZZ_ROUTE_SHUTTLETRAIN = CLAZZ_ROUTE | 0xD; // -99

    /** Landmasse des Landes: Admin Level 2 */
    public final static byte CLAZZ_BOUNDARY_LANDAREA_COUNTRY = CLAZZ_BOUNDARY | 0x1; // -63
    /** Landmasse des Bundeslandes: Admin Level 4 */
    public final static byte CLAZZ_BOUNDARY_LANDAREA_STATE = CLAZZ_BOUNDARY | 0x2; // -62
    /** Landesgrenze und gleichzeitig Landmasse : Admin Level 2 */
    public final static byte CLAZZ_BOUNDARY_ADMIN_COUNTRY_LANDAREA = CLAZZ_BOUNDARY | 0x3; // -61
    /** Landesgrenze inklusive Kuestengewaesser: Admin Level 2 */
    public final static byte CLAZZ_BOUNDARY_ADMIN_COUNTRY = CLAZZ_BOUNDARY | 0x4; // -60
    /** Bundeslandgrenze und gleichzeitig Landmasse : Admin Level 4 */
    public final static byte CLAZZ_BOUNDARY_ADMIN_STATE_LANDAREA = CLAZZ_BOUNDARY | 0x5; // -59
    /** Bundeslandgrenze inklusive Kuestengewaesser: Admin Level 4 */
    public final static byte CLAZZ_BOUNDARY_ADMIN_STATE = CLAZZ_BOUNDARY | 0x6; // -58
    /** Regierungsbezirke, sowie Metropolregionen im englischen Raum: Admin Level 5 */
    public final static byte CLAZZ_BOUNDARY_ADMIN_DISTRICT = CLAZZ_BOUNDARY | 0x7; // -57
    /** Kreisfreie Metropolen oder Kreise: Admin Level 6 */
    public final static byte CLAZZ_BOUNDARY_ADMIN_METROPOLE = CLAZZ_BOUNDARY | 0x8; // -56
    /** Deutsches Postleitzahlengebiet: Admin Level 8 */
    public final static byte CLAZZ_BOUNDARY_POSTAL = CLAZZ_BOUNDARY | 0x9; // -55
    /** Kleine und mittlere Staedte, sowie Stadtbezierke im englischen Raum: Admin Level 8 */
    public final static byte CLAZZ_BOUNDARY_ADMIN_TOWN = CLAZZ_BOUNDARY | 0xA; // -54
    /** Deutsches Postleitzahlengebiet entspricht Ortsgrenze: Admin Level 8 */
    public final static byte CLAZZ_BOUNDARY_ADMIN_TOWN_POSTAL = CLAZZ_BOUNDARY | 0xB; // -53
    /** Stadtbezirke oder Ortsteile: Admin Level 10 */
    public final static byte CLAZZ_BOUNDARY_ADMIN_SUBURB = CLAZZ_BOUNDARY | 0xC; // -52

}
