/*
 * $Id: WayTagResolver.java 5185 2020-10-28 18:38:23Z carsten $
 *
 * Copyright 2010 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.converter.tiler;

import de.cm.osm2po.model.OsmNode;

/**
 * Interface fuer Callback-Klassen, die eine Sammlung von
 * OSM-Way-Tags waehrend der Parsing-Phase auf eindeutige
 * Aussagen verdichten. Ferner wird hier entschieden,
 * ob ein Way ueberhaupt weiterverarbeitet werden soll.
 * @author (c) 2012 - Carsten Moeller - info@osm2po
 * @see DefaultWayTagResolver
 * @see TagResolver
 */
public interface WayTagResolver extends TagResolver {

    /** Einbahnstrasse : NEIN.*/
    public final int ONEWAY_NO = 0;
    /** Einbahnstrasse : JA.*/
    public final int ONEWAY_YES = 1;
    /** Einbahnstrasse : JA. Implizit aufgrund von Kreisverkehr.*/
    public final int ONEWAY_YES_ROUNDABOUT = 2;
    /** Einbahnstrasse : JA. Jedoch in entgegengesetzter Richtung.*/
    public final int ONEWAY_YES_REVERSED = -1;
    /** Einbahnstrasse : Wechselnd. (zeitabhaengig)*/
    public final int ONEWAY_YES_UNKNOWN = -2;

    /**
     * Einbahnstrasse ja/nein bzw. weitere Sonderlocken.
     * @return int Art der Einbahnstrasse.<br>
     * {@link #ONEWAY_YES}, {@link #ONEWAY_NO},
     * {@link #ONEWAY_YES_REVERSED}, {@link #ONEWAY_YES_UNKNOWN}
     */
    int getOneWayType();

    /**
     * Liefert die maximale Geschwindigkeit in Kmh.
     * Mph ist in Kmh umzurechnen.
     * @return int Geschwindigkeit in kmh
     */
    int getSpeedInKmh();

    /**
     * Liefert true, wenn es sich hier um einen OSM-Weg
     * handelt, der bei {@link OsmNode#isLevelCrossing()}
     * entkoppelt werden muss. z.B. bei Bahnuebergaengen,
     * die Schiene und Strasse verbinden.
     * @return Rail ja/nein
     */
    boolean isRail();

    /**
     * @return true: Kreisverkehr
     */
    boolean isRoundabout();

    /**
     * @return true: Keine Strasse, sondern ein Park, Gebaeude, etc.
     */
    boolean isPolygon();

}
