/*
 * $Id: BndExtension.java 4917 2018-07-24 10:29:12Z carsten $
 *
 * Copyright 2016 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.extensions;

import static de.cm.osm2po.misc.Utils.DF;
import static de.cm.osm2po.misc.Utils.roundE5;
import static java.util.Arrays.copyOf;

import java.util.HashSet;
import java.util.Set;

import de.cm.osm2po.logging.Log;
import de.cm.osm2po.misc.ZoneUtils;
import de.cm.osm2po.model.ClazzFlagPredicate;
import de.cm.osm2po.model.OsmPolygon;
import de.cm.osm2po.postp.BndExtensionBuilder;
import de.cm.osm2po.postp.BndExtensionBuilderDb;
import de.cm.osm2po.postp.GeoExtensionBuilderBase;

/**
 * Bietet komplexe Strukturen und Zugriffe fuer eine Sammlung
 * von {@link OsmPolygon}en.
 * @author (c) 2015 - Carsten Moeller - info@osm2po.de
 * @see GeoExtensionBase
 * @see BndExtensionBuilder
 * @see BndExtensionBuilderDb
 * @see GeoExtensionBuilderBase
 */
public class BndExtension extends GeoExtensionBase<OsmPolygon> {

    @Override
    public OsmPolygon[] findByCoordinate(double lat, double lon,
            ClazzFlagPredicate filter) {
        Log log = this.getLog();

        int[] polyIdxs = this.findIndexesForZone(lat, lon);
        OsmPolygon[] polygons = new OsmPolygon[8];
        int n = 0;
        if (polyIdxs != null) {
            for (int i = 0; i < polyIdxs.length; i++) {
                int idx = polyIdxs[i];
                OsmPolygon p = this.findByIndex(idx, new OsmPolygon());
                p.setIndex(idx);
                if (p.contains(lat, lon)) {
                    if (polygons.length == n)
                        polygons = copyOf(polygons, polygons.length * 2);
                    polygons[n++] = p;
                    if (log.isEnabled(Log.LEVEL_DATA)) log.data("In poly: "
                            + p.getName() + " (" + p.getClazz() + ") #" + p.getId());
                } else {
                    if (log.isEnabled(Log.LEVEL_DATA)) log.data("Not in poly: "
                        + p.getName() + " (" + p.getClazz() + ") #" + p.getId());
                }
            }
        }
        log.debug(DF(n) + " MultiPolygons contain ("
                + roundE5(lat) + " " + roundE5(lon) + ")");

        return copyOf(polygons, n);
    }

    /**
     * Findet alle Polygone aus einer Menge von Zonen.
     * @param zoneIds Array von int von Zonen NULLABLE
     * @param filter {@link ClazzFlagPredicate} NULLABLE
     * @return Array von {@link OsmPolygon} NOT NULL
     * @see ZoneUtils#createZoneId(double, double, int)
     * @see ZoneUtils#raytraceZones(double, double, double, double, int)
     */
    public final OsmPolygon[] findByZones(int[] zoneIds, ClazzFlagPredicate filter) {
        Set<Integer> polyIdxSet = new HashSet<Integer>(0x100);

        if (zoneIds != null) for (int zoneId : zoneIds) {
            int[] idxs = this.findIndexesForZone(zoneId);
            if (idxs != null) for (int idx : idxs)
                if (null == filter || filter.matches(this.findClazzByIndex(idx)))
                    polyIdxSet.add(idx);
        }

        OsmPolygon[] zonePolys = new OsmPolygon[polyIdxSet.size()];
        int n = 0;
        for (Integer polyIdx : polyIdxSet) {
            OsmPolygon multiPolygon = this.findByIndex(polyIdx.intValue(), new OsmPolygon());
            multiPolygon.setIndex(polyIdx.intValue());
            zonePolys[n++] = multiPolygon;
        }

        polyIdxSet.clear();

        return zonePolys;
    }

}