/*
 * $Id: MlgExtension.java 4714 2018-03-20 15:52:46Z carsten $
 *
 * Copyright 2011 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.extensions;

import de.cm.osm2po.primitives.InStream;
import de.cm.osm2po.primitives.OutStream;
import de.cm.osm2po.primitives.VarByteArray;

/**
 * Haelt die Daten fuer eine MultiLevelExtension.
 * @author (c) 2015 - Carsten Moeller - info@osm2po.de
 */
public class MlgExtension extends VarByteArray {
    private float maxCellSize;
    private byte nLevels;
    
    public MlgExtension(){super();}
    
    /**
     * Konstruktor.
     * @param nLevels Anzahl Levels 1-127.
     * @param maxCellSize hoechste MultiLevel-Grid-Ausdehnung.
     * @param data byte[] 
     */
    public MlgExtension(byte nLevels, float maxCellSize, byte[] data) {
        super(data);
        if (nLevels < 1) throw new IllegalArgumentException(
            "Parameter nLevels not between 1 and 127");
        if (maxCellSize < .1f || maxCellSize > 10f) throw new IllegalArgumentException(
            "MaxCellSize not between 0.1 and 10.0");
        this.maxCellSize = maxCellSize;
        this.nLevels = nLevels;
    }

    public float getMaxCellSize() {return maxCellSize;}
    public byte getNumberOfLevels() {return nLevels;}

    @Override
    public void writeToStream(OutStream outStream) {
        outStream.writeByte(this.nLevels);
        outStream.writeInt(Float.floatToIntBits(this.maxCellSize));
        super.writeToStream(outStream);
    }

    @Override
    public MlgExtension readFromStream(InStream inStream) {
        this.nLevels = inStream.readByte();
        this.maxCellSize = Float.intBitsToFloat(inStream.readInt());
        super.readFromStream(inStream);
        return this;
    }
    
    @Override
    public String toString() {
        return "nLevels:" + this.nLevels + " maxCellSize:"
            + this.maxCellSize + " levels:" + super.toString();
    }
}
