/*
 * $Id: LogConsoleWriter.java 5643 2022-04-16 22:54:08Z carsten $
 *
 * Copyright 2011 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.logging;

import static de.cm.osm2po.misc.Utils.stringCropMid;

import java.io.PrintStream;

import de.cm.osm2po.Config;

public class LogConsoleWriter implements LogWriter {

    public final static int NULL = 0;
    public final static int OUT = 1;
    public final static int ERR = 2;

    private int logLevel = Log.LEVEL_INFO;
    private PrintStream printStream;
    private boolean isLastLogProgress;
    private int lastProgressMsgLen;

    public LogConsoleWriter() {
    	this.setPrintTo(OUT);
    }

    @Override
    public LogWriter open(Config config, int index) {
        this.logLevel = Log.LEVEL_PROGRESS;
        if (config != null) {
            String strLogTo = config.getProperty("log." + index + ".to");
            if (strLogTo != null) {
                strLogTo = strLogTo.trim().toLowerCase();
                this.setPrintTo(LogConsoleWriter.NULL);
                if ("out".equals(strLogTo))
                    this.setPrintTo(LogConsoleWriter.OUT);
                if ("err".equals(strLogTo))
                    this.setPrintTo(LogConsoleWriter.ERR);
            }
            int logLevel = Log.translateLogLevel(config.getProperty(
                    "log." + index + ".level", "progress"));
            if (logLevel >= 0) this.setLogLevel(logLevel);
        }
        return this;
    }

    @Override
	public LogWriter setLogLevel(int logLevel) {
		this.logLevel = logLevel;
		return this;
	}

    @Override
    public int getLogLevel() {
        return this.logLevel;
    }

    public LogConsoleWriter setPrintTo(int printTo) {
        switch (printTo) {
        case OUT:
            this.printStream = System.out;
            break;
        case ERR:
            this.printStream = System.err;
            break;
        default:
            this.printStream = null;
            break;
        }
        return this;
    }

    @Override
    public void log(String message, int logLevel) {
        if (logLevel < this.logLevel) return;

        String msg = (Log.LEVEL_PREFIX_ARR[logLevel] + message)
        .replace("\n", "\n" + Log.LEVEL_PREFIX_ARR[Log.LEVEL_NULL]);

        if (this.printStream != null) {
            if (Log.LEVEL_PROGRESS == logLevel) {
                msg = stringCropMid(msg, 80);

                this.isLastLogProgress = true;
                this.printStream.print("\r" + msg);

                // Laengenausgleich
                int msgLen = msg.length();
                for (int i = msgLen; i < this.lastProgressMsgLen; i++)
                    this.printStream.print(" ");
                this.lastProgressMsgLen = msgLen;

            } else {
                if (isLastLogProgress) {
                    this.isLastLogProgress = false;
                    this.printStream.print("\r" + msg);

                    // Laengenausgleich
                    int msgLen = msg.length();
                    for (int i = msgLen; i < this.lastProgressMsgLen; i++)
                        this.printStream.print(" ");
                    this.lastProgressMsgLen = 0;

                    this.printStream.println();

                } else {
                    this.printStream.println(msg);
                }
            }
        }
    }

    @Override
    public void close() {this.logLevel = Integer.MAX_VALUE;}


}
