/*
 * $Id: LogJclWriter.java 4839 2018-06-01 11:17:02Z carsten $
 *
 * Copyright 2011 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.logging;

import org.apache.commons.logging.LogFactory;

import de.cm.osm2po.Config;

public class LogJclWriter implements LogWriter {

    private int logLevel;
    private org.apache.commons.logging.Log jcl;

    public LogJclWriter() {
        this.logLevel = Log.LEVEL_NULL; // delegate all
    	this.jcl = LogFactory.getLog("osm2po");
    }

    /**
     * @return neue Log-Instanz mit einem {@link LogJclWriter}
     */
    public static Log logInstance() {
        return new Log(Log.LEVEL_NULL).addLogWriter(new LogJclWriter());
    }

    @Override
    public LogWriter open(Config config, int index) {
        if (config != null) {
            Log log = config.getLog();
            if (log != null) {
                log.debug("LogWriter[" + index + "] uses "
                        + this.jcl.getClass().getName());
            }
        }
        return this;
    }

    @Override
    public void log(String message, int logLevel) {
        if (logLevel < this.logLevel) return;
        if (logLevel < Log.LEVEL_FATAL) // wegen Stacktrace
        	message = message.replace("\n", " ");
        try {
	        switch (logLevel) {
	        case Log.LEVEL_DATA: this.jcl.trace(message); break;
	        case Log.LEVEL_BUGFIX: this.jcl.trace(message); break;
	        case Log.LEVEL_DEBUG: this.jcl.debug(message); break;
	        case Log.LEVEL_WARN: this.jcl.warn(message); break;
	        case Log.LEVEL_INFO: this.jcl.info(message); break;
	        case Log.LEVEL_ERROR: this.jcl.error(message); break;
	        case Log.LEVEL_MESSAGE: this.jcl.info(message); break;
            case Log.LEVEL_ALERT: this.jcl.error(message); break;
	        case Log.LEVEL_FATAL: this.jcl.fatal(message); break;
	        }

        } catch (Exception e) {
        	throw new RuntimeException(e);
        }
    }

	@Override
	public LogWriter setLogLevel(int logLevel) {
		this.logLevel = logLevel;
		return this;
	}

	@Override public int getLogLevel() {return this.logLevel;}
    @Override public void close() {}

}
