/*
 * $Id: Code32.java 4531 2017-10-04 12:27:09Z carsten $
 *
 * Copyright 2015 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.misc;

import java.util.Arrays;

public class Code32 {
    private Code32() {/*static*/}

    private static final char ENC[] = {
        'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
        'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
        'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
        'Y', 'Z', '1', '2', '3', '4', '5', '6',
    };
    private static byte DEC[] = new byte[256];
    static {for (byte i = 0; i < ENC.length; i++) DEC[ENC[i]] = i;}

    public static String encode(byte[] bytes) {
        char[] buf = new char[bytes.length * 2];
        int n = 0;
        int ibuf = 0;
        int ibufSize = 0;
        for (int i = 0; i < bytes.length; i++) {
            int ibyte = bytes[i] & 0xff;
            ibyte <<= ibufSize;
            ibuf |= ibyte;
            ibufSize += 8;
            while (ibufSize >= 5) {
                int x = ibuf & 0x1f;
                buf[n++] = ENC[x];
                ibuf >>>= 5;
                ibufSize -= 5;
            }
        }
        if (ibufSize != 0) {
            buf[n++] = ENC[ibuf];
        }
        return String.valueOf(buf, 0, n);
    }

    public static byte[] decode(String s) {
        byte[] buf = new byte[s.length()];
        int n = 0;
        int ibuf = 0;
        int ibufSize = 0;
        for (int i = 0; i < s.length(); i++) {
            int ibyte = DEC[s.charAt(i)];
            ibyte <<= ibufSize;
            ibuf |= ibyte;
            ibufSize += 5;
            if (ibufSize >= 8) {
                buf[n++] = (byte) ibuf;
                ibuf >>>= 8;
                ibufSize -= 8;
            }
        }
        return Arrays.copyOf(buf, n);
    }

}
