/*
 * $Id: DataUrlParser.java 5000 2019-11-11 18:32:11Z carsten $
 *
 * Copyright 2019 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.misc;

import java.nio.charset.Charset;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class DataUrlParser {

    private DataUrlParser() {/*STATIC*/}

    public final static class DataUrl {
        public final String mimeType;
        public final String mimeSubType;
        public final Object object;
        public DataUrl(String mimeType, String mimeSubType,
                String charset, String encoding, String payload) {
            this.mimeType = mimeType;
            this.mimeSubType = mimeSubType;

            if ("text".equalsIgnoreCase(mimeType)) {
                Charset ch = (null == charset) ? Utils.UTF8 : Charset.forName(charset);
                this.object = new String(payload.getBytes(), ch);
            } else if ("base64".equalsIgnoreCase(encoding)) { // base64-String unsupported
                this.object = Base64.decode(payload);
            } else {
                throw new RuntimeException("Unsupported DataUrl-Encoding " + encoding);
            }
        }
    }

    public static DataUrl parseDataUrl(final String dataUrl) throws Exception {
        if (null == dataUrl) return null;
        if (!dataUrl.startsWith("data:")) return null; // Quick Pre-Check

        int plx = dataUrl.indexOf(',');
        if (plx == -1) return null;

        String meta = dataUrl.substring(0, plx);
        String payload = dataUrl.substring(plx +1);

        // 1:image 2:jpeg 3:utf-8 4:base64
        final String regex = "^data:(\\w+)/(\\w+);(?:charset=([a-zA-Z_0-9\\-]+))?;?(\\w+)?$";
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(meta);
        if (!matcher.matches()) return null;

        return new DataUrl(matcher.group(1), matcher.group(2),
                matcher.group(3), matcher.group(4), payload);
    }

}
