/*
 * $Id: ClazzFlagPredicate.java 5239 2021-01-19 18:58:24Z carsten $
 *
 * Copyright 2018 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.model;

import static de.cm.osm2po.misc.ArrayUtils.csvBytesToBooleanArray;

/**
 * Ein Praedikat (Filter) fuer clazz und flags.
 * @author (c) 2018 - Carsten Moeller - info@osm2po
 */
public final class ClazzFlagPredicate {
    private int flags;
    private boolean[] clazzes;

    /**
     * CSV-/String-Konstruktor.
     * @param strFlags Flags als Integer-String, NULLABLE
     * @param strClazzes Array von Bytes als CSV, NULLABLE
     */
    public ClazzFlagPredicate(String strFlags, String strClazzes) {
        // Auch Ranges ala 1-3,7,11-13 moeglich
        this.clazzes = csvBytesToBooleanArray(strClazzes);
        if (strFlags != null) {
            this.flags = Integer.valueOf(strFlags.trim());
        }
    }

    /**
     * Konstruktor.
     * @param flags int
     * @param clazzes optionales Array von clazzes (bytes)
     */
    public ClazzFlagPredicate(int flags, byte... clazzes) {
        this.flags = flags;
        if (clazzes != null && clazzes.length > 0) {
            this.clazzes = new boolean[256];
            for (byte clazz : clazzes) {
                int index = clazz + 128;
                this.clazzes[index] = true;
            }
        }
    }

    public final boolean matches(OsmObject osmObject) {
        return this.matches(osmObject.getClazz())
            && this.matches(osmObject.getFlags());
    }

    public final boolean matches(int flags) {
        return (0 == this.flags || (flags & this.flags) != 0);
    }

    public final boolean matches(byte clazz) {
        return (null == this.clazzes || this.clazzes[clazz + 128]);
    }

}
