/*
 * $Id: Node.java 4357 2017-02-05 21:12:54Z carsten $
 *
 * Copyright 2011 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.model;

import de.cm.osm2po.primitives.InStream;
import de.cm.osm2po.primitives.OutStream;

/**
 * Erweitert {@link OsmNode} um z.B. {@link Membership}
 * und weitere von OSM nicht unterstuetzte Attribute.
 * @see OsmNode
 * @see Way
 * @see Vertex
 * @author (c) 2012 - Carsten Moeller - info@osm2po
 */
public class Node extends OsmNode {
    
    private Membership[] memberships;

    /** Std.CTOR. */
    public Node() {super();}
    
    public Node(double lat, double lon) {
        super(lat, lon);
    }
    
    public void setMemberships(Membership[] memberships) {
        this.memberships = memberships;
    }
    
    public Membership[] getMemberships() {
        return this.memberships;
    }
    
    @Override
    public void clear() {
        super.clear();
        this.memberships = null;
    }
    
    @Override
    public String toString() {
        String s = super.toString();
        if (this.memberships != null) {
            s += " Memberships[" + this.memberships.length + "]:";
            for (int i = 0; i < this.memberships.length; i++) {
                s += " (" + this.memberships[i].toString() + ")";
            }
        }
        return s;
    }
    
    public void writeToOsmNodeStream(OutStream outStream) {
        super.writeToStream(outStream);
    }
    
    @Override
    public void writeToStream(OutStream outStream) {
        super.writeToStream(outStream);
        
        int nMemberships = (null == this.memberships) ? 0 : this.memberships.length;
        if (nMemberships > 0xFF) nMemberships = 0xFF; // Byte absichern. 255 max.
        // TODO nRelation mit VarNum und InternalFlags aus OsmNode
        // Hinweis: Anzahl Relationen > 255 hoechst unwahrscheinlich!
        outStream.writeByte((byte) nMemberships);
        if (nMemberships > 0) {
            for (int i = 0; i < nMemberships; i++) {
                Membership r = this.memberships[i];
                r.writeToStream(outStream);
            }
        }
    }

    /**
     * Upcast-Reader von {@link OsmNode} nach {@link Node}.
     * @param inStream {@link InStream} von OsmNodes.
     * @return this Node 
     */
    public Node readFromOsmNodeStream(InStream inStream) {
        this.memberships = null;
        super.readFromStream(inStream);
        return this;
    }
    
    @Override
    public Node readFromStream(InStream inStream) {
        super.readFromStream(inStream);
        // Aus Platzgruenden wird Byte verwendet. Absichern!
        int nMemberships = inStream.readByte() & 0xFF;
        if (nMemberships > 0) {
            this.memberships = new Membership[nMemberships];
            for (int i = 0; i < nMemberships; i++) {
                this.memberships[i] = new Membership();
                this.memberships[i].readFromStream(inStream);
            }
        }
        return this;
    }
    
}
