/*
 * $Id: OsmObject.java 5239 2021-01-19 18:58:24Z carsten $
 *
 * Copyright 2015 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.model;

import de.cm.osm2po.converter.tiler.WayTagResolver;
import de.cm.osm2po.primitives.Var;
import de.cm.osm2po.primitives.VarString;

/**
 * Interface fuer Klassen, die OSM-Objekte oder hiervon
 * abgeleitete Varianten repraesentieren.
 * Folgende Attribute muessen bereitgestellt werden:
 * <ol>
 * <li><strong>id</strong> : <code>long</code></li>
 * <li><strong>clazz</strong> : <code>byte</code></li>
 * <li><strong>flags</strong> : <code>int</code></li>
 * <li><strong>name</strong>  : <code>{@link VarString}</code></li>
 * </ol>
 *
 * @author (c) 2015 - Carsten Moeller - info@osm2po.de
 * @see OsmNode
 * @see OsmWay
 * @see OsmRelation
 */
public interface OsmObject {
    /** @return long OsmId  */
    long getId();
    /** @return byte vom {@link WayTagResolver} ermittelter Typ */
    byte getClazz();
    /** @return int zusaetzliche Flags oder benutzerdefinierter Wert */
    int getFlags();
    /** @return {@link VarString} Name des Osm-Objekts als UTF-8 */
    VarString getName();
    /** @return {@link Var} fuer komplexe benutzerdefinierte Werte */
    Var<?> getCustom();
    /** @param index int Temporaerer Index bei Verwendung in Containern */
    void setIndex(int index);
    int getIndex();
}
