/*
 * $Id: WaySegment.java 4390 2017-03-04 07:50:45Z carsten $
 *
 * Copyright 2011 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.model;

import de.cm.osm2po.converter.Segmenter;
import de.cm.osm2po.primitives.InStream;
import de.cm.osm2po.primitives.OutStream;
import de.cm.osm2po.primitives.Var;

/**
 * Repraesentiert ein Weg-WaySegment nach der Verarbeitung
 * mittels {@link Segmenter}. Ein Weg-WaySegment vereinigt
 * eine Teilkette von Knoten-Indizes, den Ids der Start- und Ziel-
 * Vertices, sowie die neu vergebenen WaySegment-Id selbst.
 * @author (c) 2012 - Carsten Moeller - info@osm2po
 * @see SegmentedWay
 */
public class WaySegment implements Var<WaySegment> {
    private int id;
    private int sourceId;
    private int targetId;
    private int sourceIdx;
    private int targetIdx;
    
    public int getId() {return this.id;}
    public int getSourceId() {return this.sourceId;} // VertexId
    public int getTargetId() {return this.targetId;} // VertexId
    public int getSourceIdx() {return sourceIdx;} // Index auf Knoten
    public int getTargetIdx() {return targetIdx;} // Index auf Knoten

    // Fuer nachtraegliches Setzen durch QuadTiling
    public void setSourceId(int sourceId) {this.sourceId = sourceId;}
    public void setTargetId(int targetId) {this.targetId = targetId;}

    /** STDCTOR. */
    public WaySegment() {}
    
    /**
     * CTOR.
     * @param id SegmentId not 0
     * @param sourceId Vertex-SourceId not 0
     * @param targetId Vertex-TargetId not 0
     * @param sourceNodeIdx int Index des SourceNodes im {@link Way}
     * @param targetNodeIdx int Index des TargetNodes im {@link Way}
     */
    public WaySegment(int id, int sourceId, int targetId, int sourceNodeIdx, int targetNodeIdx) {
        this.id = id;
        this.sourceId = sourceId;
        this.targetId = targetId;
        this.sourceIdx = sourceNodeIdx;
        this.targetIdx = targetNodeIdx;
        if (0 == id || 0 == sourceId || 0 == targetId || (targetNodeIdx - sourceNodeIdx == 0))
            throw new RuntimeException("Invalid WaySegment \n" + this.toString());
    }
    
    @Override
    public String toString() {
        return "Id:" + this.id
            + " SourceId:" + this.sourceId + " TargetId:" + this.targetId
            + " SourceIdx:" + this.sourceIdx + " TargetIdx:" + this.targetIdx;
    }
    
    @Override
    public WaySegment readFromStream(InStream inStream) {
        this.id = inStream.readInt();
        this.sourceId = inStream.readInt();
        this.targetId = inStream.readInt();
        this.sourceIdx = inStream.readInt();
        this.targetIdx = inStream.readInt();
        return this;
    }
    
    @Override
    public void writeToStream(OutStream outStream) {
        outStream.writeInt(this.id);
        outStream.writeInt(this.sourceId);
        outStream.writeInt(this.targetId);
        outStream.writeInt(this.sourceIdx);
        outStream.writeInt(this.targetIdx);
    }

}
