/*
 * $Id: BndExtensionBuilder.java 5502 2022-02-06 15:25:33Z carsten $
 *
 * Copyright 2016 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.postp;

import static de.cm.osm2po.misc.ArrayUtils.intArrayConcat;
import static de.cm.osm2po.misc.Utils.DF;
import static de.cm.osm2po.misc.ZoneUtils.createZoneIdsBox;

import de.cm.osm2po.Config;
import de.cm.osm2po.converter.joiner.JoinedWaysIterator;
import de.cm.osm2po.extensions.BndExtension;
import de.cm.osm2po.logging.Log;
import de.cm.osm2po.model.LatLons;
import de.cm.osm2po.model.Membership;
import de.cm.osm2po.model.OsmPolygon;
import de.cm.osm2po.model.OsmPolygonBuilder;
import de.cm.osm2po.model.Way;
import de.cm.osm2po.primitives.Var;
import de.cm.osm2po.routing.Graph;

/**
 * Erweitert die {@link Graph}-Datei um Boundaries (Multipolygone),
 * die von osm2po selbst verarbeitet wurden. Diese haengen als
 * {@link Membership}-Fragmente an den {@link Way}s.
 *
 * @author (c) 2016 - Carsten Moeller, info@osm2po.de
 * @see GeoExtensionBuilderBase
 */
public class BndExtensionBuilder
extends GeoExtensionBuilderBase<OsmPolygon> {

    private Config config;

    @Override
    public Class<? extends Var<?>> forExtensionClass() {
        return BndExtension.class;
    }

    @Override
    public void run(Config config, int index) throws Exception {
        this.config = config;
        super.run(config, index);
    }

    @Override
    public Iterable<OsmPolygon> getSource() {
        OsmPolygonBuilder polygonBuilder = new OsmPolygonBuilder(log);

        int nWays = 0;
        for (Way way : new JoinedWaysIterator(this.config)) {
            polygonBuilder.collectMemberships(way);
            if (++nWays % 25000 == 0)
                log.progress(DF(nWays) + " ways collected");
        }
        log.info(DF(nWays) + " ways collected");

        return polygonBuilder;
    }

    @Override
    public int[] zoneIdsOf(OsmPolygon polygon, byte zoneSize) {
        int[] zoneIds = null;
        // ZoneIds der OuterPolygons ermitteln (Plural!)
        for (LatLons[] polyCoords : polygon.getPolygons()) {
            try {
                zoneIds = intArrayConcat( // polygon[0] ist Outer
                    zoneIds, createZoneIdsBox(polyCoords[0].getCoords(), zoneSize));
            } catch (Throwable t) {
                // TODO ungueltige Polygone frueher absichern
                log.error(t.toString());
                log.debug(polygon.toString() + "\n" + Log.translateError(t, 3));
            }
        }
        return zoneIds;
    }

}
