/*
 * $Id: JoinRereader.java 5123 2020-10-19 19:43:39Z carsten $
 *
 * Copyright 2017 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.postp;

import de.cm.osm2po.Config;
import de.cm.osm2po.converter.GraphBuilder;
import de.cm.osm2po.converter.Joiner;
import de.cm.osm2po.converter.Segmenter;
import de.cm.osm2po.converter.joiner.JoinFilter;
import de.cm.osm2po.converter.joiner.JoinedWaysIterator;
import de.cm.osm2po.logging.Log;
import de.cm.osm2po.model.Way;

/**
 * Ein Postprozessor, der alle Wege aus dem Join-Lauf mittels
 * {@link JoinedWaysIterator} durchlaeuft und die {@link Way}s an einen
 * {@link JoinFilter} weiterreicht. Dieser kann entweder mittels
 * <code>postp.X.joinFilter.class</code> oder einfach global mittels
 * <code>joinFilter.class</code> gesetzt werden.
 * <br><br><strong>Wichtig:</strong><br>
 * Objwohl dies ein {@link PostProcessor} ist, macht es nur
 * Sinn, diesen nach dem {@link Joiner}-Lauf und noch vor dem
 * {@link Segmenter} bzw. {@link GraphBuilder} zu starten.
 *
 * @author (c) 2017 - Carsten Moeller - info@osm2po.de
 */
public class JoinRereader implements PostProcessor {

    @SuppressWarnings("unchecked")
    @Override
    public void run(Config config, int index) throws Exception {
        String prefix = null;

        Class<? extends JoinFilter> joinFilterClass = config.getJoinFilterClass();
        final String strJoinFilterClass = config.getProperty(
                Config.POSTP(index) + ".joinFilter.class");
        if (strJoinFilterClass != null) {
            joinFilterClass = (Class<? extends JoinFilter>) Config.loadClass(strJoinFilterClass);
            prefix = Config.POSTP(index);
        }

        final JoinFilter joinFilter = joinFilterClass.getDeclaredConstructor().newInstance();
        final Log log = config.getLog();

        log.info("Filtering Ways from '" + config.getWorkDir()
            + "' with '"  + joinFilter.getClass().getSimpleName() + "'");

        joinFilter.open(config, prefix);
        for (Way way : new JoinedWaysIterator(config)) {
            joinFilter.validate(way);
        }
        joinFilter.close();
    }

}
