/*
 * $Id: PoiExtensionBuilder2po.java 4714 2018-03-20 15:52:46Z carsten $
 *
 * Copyright 2016 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.postp;

import static de.cm.osm2po.misc.ZoneUtils.createZoneId;
import static de.cm.osm2po.misc.ZoneUtils.expandZone;

import java.io.File;
import java.util.Iterator;

import de.cm.osm2po.Config;
import de.cm.osm2po.extensions.PoiExtension;
import de.cm.osm2po.model.OsmNode;
import de.cm.osm2po.primitives.InStream;
import de.cm.osm2po.primitives.InStreamDisk;
import de.cm.osm2po.primitives.Var;
import de.cm.osm2po.primitives.VarTypeDesk;
import de.cm.osm2po.routing.Graph;

/**
 * Erstellt eine {@link PoiExtension} aus einer 2po-Datei
 * von {@link OsmNode}s und/oder schreibt diese in eine
 * {@link Graph}-Datei.
 *
 * @author (c) 2017 - Carsten Moeller, info@osm2po.de
 * @see GeoExtensionBuilderBase
 */
public class PoiExtensionBuilder2po extends GeoExtensionBuilderBase<OsmNode> {

    private InStream inStream;

    @Override
    public Class<? extends Var<?>> forExtensionClass() {
        return PoiExtension.class;
    }

    @Override
    public void run(Config config, int index) throws Exception {
        // TODO Fehler ordentlich loggen
        final String prep = Config.POSTP(index) + ".";
        File file = new File(config.getProperty(prep + "file"));
        this.inStream = new InStreamDisk(file);
        byte varType = this.inStream.readByte();
        if (!(VarTypeDesk.typeIdOf(OsmNode.class) == varType))
            throw new RuntimeException("Unexpected VarType " + varType
                    + " in " + file.getAbsolutePath());
        super.run(config, index);
        this.inStream.close();
    }

    @Override
    public Iterable<OsmNode> getSource() {
        return new Iterable<OsmNode>() {
            @Override
            public Iterator<OsmNode> iterator() {
                return new Iterator<OsmNode>() {
                    @Override
                    public boolean hasNext() {
                        return !inStream.isEof();
                    }

                    @Override
                    public OsmNode next() {
                        return new OsmNode().readFromStream(inStream);
                    }

                    @Override
                    public void remove() {
                        throw new UnsupportedOperationException();
                    }
                };
            }
        };
    }

    @Override
    public int[] zoneIdsOf(OsmNode osmNode, byte zoneSize) {
        int zoneId = createZoneId(osmNode.getLat(), osmNode.getLon(), zoneSize);
        int[] zoneIds = expandZone(zoneId, 1, zoneSize);
        return zoneIds;
    }

}
