/*
 * $Id: Bits.java 3085 2014-05-01 20:04:15Z cmindividual $
 *
 * Copyright 2010 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;

import java.io.DataInputStream;

import de.cm.osm2po.misc.WkbUtils;


/**
 * Diese Klasse bietet statische Hilfsmethoden um
 * Standarddatentypen wie int oder float direkt
 * in byte-Arrays zu schreiben bzw. aus ihnen zu lesen.
 * Der Code ist eine Kopie aus der versteckten
 * java.io.Bits-Klasse. Die Verwendung dieser direkten
 * Methoden macht den Einsatz von {@link DataInputStream}
 * und deren Verwandte ueberfluessig und ist dadurch
 * etwa doppelt so schnell.
 * Die Byte-Anordnung ist XDR, nich NDR (siehe {@link WkbUtils})
 * 
 * @author (c) 2012 - Carsten Moeller - info@osm2po
 */
public class Bits {

    /** STATIC. */
    private Bits() {}
    
    /**
     * Liest ein <strong>short</strong> aus einem ByteArray.
     * @param b byte[]
     * @param off int Offset
     * @return short
     */
    public static short getShort(byte[] b, int off) {
        return (short) (((b[off + 1] & 0xFF) << 0) + 
                ((b[off + 0]) << 8));
    }
    
    /**
     * Liest ein <strong>int</strong> aus einem ByteArray.
     * @param b byte[]
     * @param off int Offset
     * @return int
     */
    public static int getInt(byte[] b, int off) {
        return ((b[off + 3] & 0xFF) << 0) +
        ((b[off + 2] & 0xFF) << 8) +
        ((b[off + 1] & 0xFF) << 16) +
        ((b[off + 0]) << 24);
    }

    /**
     * Liest ein <strong>long</strong> aus einem ByteArray.
     * @param b byte[]
     * @param off int Offset
     * @return int
     */
    public static long getLong(byte[] b, int off) {
        return ((b[off + 7] & 0xFFL) << 0) +
        ((b[off + 6] & 0xFFL) << 8) +
        ((b[off + 5] & 0xFFL) << 16) +
        ((b[off + 4] & 0xFFL) << 24) +
        ((b[off + 3] & 0xFFL) << 32) +
        ((b[off + 2] & 0xFFL) << 40) +
        ((b[off + 1] & 0xFFL) << 48) +
        (((long) b[off + 0]) << 56);
    }
    
    
    /**
     * Schreibt ein <strong>short</strong> in ein ByteArray.
     * @param b byte[] ByteArray
     * @param off Offset
     * @param val short
     */
    public static void putShort(byte[] b, int off, short val) {
        b[off + 1] = (byte) (val >>> 0);
        b[off + 0] = (byte) (val >>> 8);
    }

    /**
     * Schreibt ein <strong>int</strong> in ein ByteArray.
     * @param b byte[] ByteArray
     * @param off Offset
     * @param val int
     */
    public static void putInt(byte[] b, int off, int val) {
        b[off + 3] = (byte) (val >>> 0);
        b[off + 2] = (byte) (val >>> 8);
        b[off + 1] = (byte) (val >>> 16);
        b[off + 0] = (byte) (val >>> 24);
    }
    
    /**
     * Schreibt ein <strong>long</strong> in ein ByteArray.
     * @param b byte[] ByteArray
     * @param off Offset
     * @param val float
     */
    public static void putLong(byte[] b, int off, long val) {
        b[off + 7] = (byte) (val >>> 0);
        b[off + 6] = (byte) (val >>> 8);
        b[off + 5] = (byte) (val >>> 16);
        b[off + 4] = (byte) (val >>> 24);
        b[off + 3] = (byte) (val >>> 32);
        b[off + 2] = (byte) (val >>> 40);
        b[off + 1] = (byte) (val >>> 48);
        b[off + 0] = (byte) (val >>> 56);
    }
    
    
}
