/*
 * $Id: InStream.java 5211 2020-11-04 20:28:21Z carsten $
 *
 * Copyright 2010 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;

/**
 * Schnittstelle fuer alle Eingabe-Datenstroeme.
 *
 * @see OutStream
 * @see RandomAccess
 * @see InStreamDisk
 * @see InStreamAdapter
 *
 * @author (c) 2012 - Carsten Moeller - info@osm2po
 */
public interface InStream {

    /** @return byte an der aktuellen Lese-Position. */
    byte readByte();

    /**
     * Liest ab der aktuellen Lese-Position Bytes
     * in den Puffer bis dieser voll ist oder EOF
     * erreicht wurde.
     * @param buffer byte[] Puffer.
     * @return int Anzahl gelesener Bytes &gt;= 0.
     * <br><b>Wichtig:</b><br>
     * Aufgrund der Eigenheiten der Java-IO ist nicht garantiert,
     * dass der Buffer immer vollstaendig gefuellt wird.
     * Es koennen auch mal weniger Bytes geladen werden, obwohl
     * theoretisch noch Platz gewesen waere.
     */
    int readBytes(byte[] buffer);

    /** @return short an der aktuellen Lese-Position. */
    short readShort();

    /** @return long an der aktuellen Lese-Position. */
    long readLong();

    /** @return int an der aktuellen Lese-Position. */
    int readInt();

    /** @return End Of File ja/nein */
    boolean isEof();

    /** @return aktuelle Leseposition. */
    long getReadIndex();

    /** @param n long Ueberspringt n Bytes im Eingabestrom*/
    void skip(long n);

    /** Close this stream. */
    void close();

}
