/*
 * $Id: InStreamDisk.java 4569 2017-12-03 12:52:13Z carsten $
 *
 * Copyright 2010 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;

import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;

public class InStreamDisk implements InStream {

    private DataInputStream dis;
    private long readIndex;
    private long fileLen;

    public InStreamDisk(File file) {
        try {
            this.fileLen = file.length();
            this.dis = new DataInputStream(new BufferedInputStream(
                    new FileInputStream(file), 0x8000 /*32k*/));
            this.readIndex = 0;

        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public void close() {
        try {
            if (this.dis != null) this.dis.close();
            this.readIndex = 0;

        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public byte readByte() {
        try {
            this.readIndex += 1;
            return this.dis.readByte();

        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public short readShort() {
        try {
            this.readIndex += 2;
            return this.dis.readShort();

        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public int readInt() {
        try {
            this.readIndex += 4;
            return this.dis.readInt();

        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public long readLong() {
        try {
            this.readIndex += 8;
            return this.dis.readLong();

        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public boolean isEof() {
        // Do not trust InputStream.available() !
        return this.readIndex >= this.fileLen;
    }

    @Override
    public long getReadIndex() {
        return this.readIndex;
    }

    @Override
    public int readBytes(byte[] buffer) {
        int nRead = 0;
        try {
            nRead = this.dis.read(buffer);
            if (nRead > 0) {
                this.readIndex += nRead;
            } else {
                nRead = 0;
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        return nRead;
    }

    @Override
    public void skip(long n) {
        try {
            while (n > 0) {
                long nSkipped = this.dis.skip(n);
                n -= nSkipped;
                this.readIndex += nSkipped;
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

}
