/*
 * $Id: IndexRaw.java 5239 2021-01-19 18:58:24Z carsten $
 *
 * Copyright 2012 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;

import static de.cm.osm2po.misc.Utils.createTmpFile;

import java.io.File;

import de.cm.osm2po.logging.Log;

/**
 * Diese Implementation von {@link Index} erstellt den Index
 * ohne die uebergebenen Daten vorher zu optimieren.
 *
 * @author (c) 2015 - Carsten Moeller - info@osm2po.de
 * @see IndexSeq
 * @see IndexSeqOpt
 * @param <T> Class of Var
 */
public class IndexRaw<T extends Var<?>> implements Index<T> {
    private File varFile;
    private File idxFile;
    private long offsIdx;
    private long offsVar;

    protected RandomRead rr;
    protected int size;
    protected OutStream osVar;
    protected OutStream osIdx;
    protected boolean buildMode;

    @Override
    public Index<T> create(File workDir, Log log) {
        this.buildMode = true;
        this.varFile = createTmpFile(workDir);
        this.idxFile = createTmpFile(workDir);
        this.osVar = new OutStreamDisk(this.varFile);
        this.osIdx = new OutStreamDisk(this.idxFile);
        return this;
    }

    @Override
    public void connect(RandomRead rr) {
        this.size = rr.readInt();
        this.offsIdx = rr.getReadIndex();
        this.offsVar = this.offsIdx + (this.size << 3); // 1long=8bytes
        this.rr = rr;
    }

    @Override
    public int getIndexSize() {return this.size;}

    @Override
    public T fetch(int idx, T var) {
		if (this.buildMode) throw new IllegalStateException("Not in Runtime-Mode");
		if (idx < 0 || idx >= this.size) return null;
		synchronized (this.rr) {
			this.rr.setReadIndex(this.offsIdx + (idx << 3));
			long ptr = this.rr.readLong();
			this.rr.setReadIndex(this.offsVar + ptr);
			var.readFromStream(this.rr);
		}
		return var;
    }

    @Override
	public int push(T var) {
		if (!this.buildMode) throw new IllegalStateException("Not in Build-Mode");
		long ptr = this.osVar.getWriteIndex();
		var.writeToStream(this.osVar);
	    this.osIdx.writeLong(ptr);
	    int idx = this.size++;
	    return idx;
	}

    @Override
	public void build(OutStream os) {
		if (!this.buildMode) throw new IllegalStateException("Not in Build-Mode");
		InStream is;
		this.osIdx.close();
		this.osVar.close();
		os.writeInt(this.size);
		byte[] buf = new byte[0x1000];
    	is = new InStreamDisk(this.idxFile);
    	while (!is.isEof()) {int n = is.readBytes(buf);	os.writeBytes(buf, n);}
		is.close();
		this.idxFile.delete();
		is = new InStreamDisk(this.varFile);
    	while (!is.isEof()) {int n = is.readBytes(buf);	os.writeBytes(buf, n);}
		is.close();
		this.varFile.delete();
		this.buildMode = false;
	}

}
