/*
 * $Id: IndexSeq.java 5244 2021-02-10 19:53:01Z carsten $
 *
 * Copyright 2012 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;

/**
 * Diese Implementation von {@link Index} erbt von
 * {@link IndexRaw} und benutzt so simple longs als Indizes.
 * Allerdings werden hier {@link Var}-Werte vor dem Herausschreiben
 * mit dem Vorgaenger verglichen und bei Wiederholungen lediglich
 * die Indizes geschrieben.
 * <br><br><b>Wichtig:</b><br>
 * Damit dieser Mechanismus sinnvoll funktioniert, muessen verwendete
 * {@link Var}-Klassen {@link #equals(Object)} ueberschreiben.
 *
 * @author (c) 2015 - Carsten Moeller - info@osm2po.de
 * @see IndexSeqOpt
 * @param <T> Class of Var
 */
public class IndexSeq<T extends Var<?>> extends IndexRaw<T> {
	private T var;
    private int nPending;

    @Override
	public int push(T var) {
		if (!this.buildMode) throw new IllegalStateException("Not in Build-Mode");
		if (null == this.var) {
	        try {
                // called once and checks for an existing equals method
	            var.getClass().getDeclaredMethod("equals", Object.class);
	        } catch (Exception e) {
	            throw new IllegalArgumentException(
	                    "Class '" + var.getClass().getSimpleName()
	                    + "' does not provide equals()", e);
	        }
		    this.var = var;
		}
		if (this.var.equals(var)) this.nPending++;
		else {
			long ptr = this.osVar.getWriteIndex();
			this.var.writeToStream(this.osVar);
			for (int i = 0; i < this.nPending; i++) {
			    this.osIdx.writeLong(ptr);
			    this.size++;
			}
			this.var = var;
            this.nPending = null == var ? 0 : 1;
		}
		return this.size + nPending -1;
	}

    @Override
	public void build(OutStream os) {
        if (!this.buildMode) throw new IllegalStateException("Not in Build-Mode");
        if (this.nPending > 0) this.push(null); // Flush Buffer
        super.build(os);
	}

}
