/*
 * $Id: KeyVal.java 5545 2022-02-25 17:34:02Z carsten $
 *
 * Copyright 2021 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;

/**
 * Klassensammlung von Key-Value-Pairs fuer primitive
 * Datentypen wie klein int oder long.
 * @see IntIntHashMap
 * @see LongLongHashMap
 * @see IntFloatHashMap
 * @see IntTHashMap
 * @author (c) 2013/2014 - Carsten Moeller - info@osm2po
 */
public class KeyVal {
    private KeyVal(/*STATIC*/) {}

    public final static class KV<K, V> {
        public final K key;
        public final V val;
        public KV(K key, V val) {this.key = key; this.val = val;}
        @Override public String toString() {return "key:" + key + ", val:"+ val;}
    }

    public final static class IntV<V> {
        public final int key;
        public final V val;
        public IntV(int key, V val) {this.key = key; this.val = val;}
        @Override public String toString() {return "key:" + key + ", val:"+ val;}
    }

    public final static class IntInt {
        public final int key;
        public final int val;
        public IntInt(int key, int val) {this.key = key; this.val = val;}
        @Override public String toString() {return "key:" + key + ", val:"+ val;}
    }

    public final static class IntFloat {
        public final int key;
        public final float val;
        public IntFloat(int key, float val) {this.key = key; this.val = val;}
        @Override public String toString() {return "key:" + key + ", val:"+ val;}
    }

    public final static class IntDouble {
        public final int key;
        public final double val;
        public IntDouble(int key, double val) {this.key = key; this.val = val;}
        @Override public String toString() {return "key:" + key + ", val:"+ val;}
    }

    public final static class LongLong {
        public final long key;
        public final long val;
        public LongLong(long key, long val) {this.key = key; this.val = val;}
        @Override public String toString() {return "key:" + key + ", val:"+ val;}
    }

}
