/*
 * $Id: OutStreamDisk.java 4287 2016-08-03 09:39:31Z carsten $
 *
 * Copyright 2010 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;

/**
 * @author (c) 2014 - Carsten Moeller, info@osm2po.de
 */
public class OutStreamDisk implements OutStream, Concatenable {
    
    private DataOutputStream dos;
    private long writeIndex;
    
    public OutStreamDisk(File file) {
        try {
            this.dos = new DataOutputStream(new BufferedOutputStream(
                    new FileOutputStream(file), 0x8000 /*32k*/));
            this.writeIndex = 0;
            
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
    
    @Override
    public void close() {
        try {
            if (this.dos != null) this.dos.close();
            this.writeIndex = 0;

        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public void concatFile(File file, long skip) {
        try {
        	byte[] buffer = new byte[4096];
            InputStream is = new BufferedInputStream(
                    new FileInputStream(file), 0x8000 /*32k*/);
            is.skip(skip);
            for (int nRead = is.read(buffer); nRead >= 0; nRead = is.read(buffer)) {
	            this.dos.write(buffer, 0, nRead);
	            this.writeIndex += nRead;
            }
            is.close();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public void writeByte(byte b) {
        try {
            this.writeIndex += 1;
            this.dos.writeByte(b);
            
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public void writeShort(short s) {
        try {
            this.writeIndex += 2;
            this.dos.writeShort(s);
            
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public void writeInt(int i) {
        try {
            this.writeIndex += 4;
            this.dos.writeInt(i);
            
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public void writeLong(long l) {
        try {
            this.writeIndex += 8;
            this.dos.writeLong(l);
            
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public long getWriteIndex() {
        return this.writeIndex;
    }

    @Override
    public void writeBytes(byte[] bytes, int len) {
        try {
            this.dos.write(bytes, 0, len);
            this.writeIndex += len;
            
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

}
