/*
 * $Id: RandomAccess.java 4370 2017-02-14 19:50:44Z carsten $
 *
 * Copyright 2010 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;

/**
 * Dieses Interface erweitert {@link OutStream}
 * und {@link InStream} um die Option
 * Daten nicht nur sequentiell zu schreiben oder zu lesen,
 * sondern auch wahlfrei die Position fuer die naechste Lese- oder
 * Schreibaktion zu setzen.<br>
 * <br><b>Wichtig</b><br>
 * Der Gebrauch der setIndex() oder getIndex()-Methoden
 * ist nicht thread-safe!<br>
 * Folglich muss der Aufrufer fuer eine Synchronisierung sorgen.<br>
 * Beispiel:<br><br>
 * <code> 
 * RandomAccess ra = new ...<br>
 * synchronized (ra) {<br>
 *   ra.setReadIndex(long index);<br>
 *   ra.readInt();<br>
 * }<br>
 * </code>
 * @see RandomRead
 * @see RandomWrite
 * @see RandomAccessMemory
 * @see RandomAccessDisk
 * @see RandomAccessArray
 * 
 * @author (c) 2013 - Carsten Moeller - info@osm2po
 */
public interface RandomAccess extends RandomRead, RandomWrite {
}
