/*
 * $Id: RandomWrite.java 4353 2017-02-05 11:48:42Z carsten $
 *
 * Copyright 2010 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;

/**
 * Dieses Interface erweitert {@link OutStream} um die Option
 * Daten nicht nur sequentiell zu schreiben,
 * sondern auch wahlfrei die Position fuer die Schreibaktion
 * zu setzen.<br>
 * <br><b>Wichtig</b><br>
 * Der Gebrauch der {@link #setWriteIndex(long)}-Methoden
 * ist nicht thread-safe!<br>
 * Folglich muss der Aufrufer fuer eine Synchronisierung sorgen.
 * 
 * @see RandomAccessMemory
 * @see RandomAccessDisk
 * 
 * @author (c) 2013 - Carsten Moeller - info@osm2po
 */
public interface RandomWrite extends OutStream {

    /** Setzt die Schreibposition wieder ans Ende. */
    void setWriteIndexEof();

    /** @param index Setzt die aktuelle Schreibposition. */
    void setWriteIndex(long index);

    /** Setzt zurueck und gibt ggf. verbrauchte Resourcen wieder frei. */
    void clear();

}
