/*
 * $Id: Var.java 5243 2021-02-09 21:03:31Z carsten $
 *
 * Copyright 2010 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;

import java.io.Serializable;

/**
 * Instanzen dieses Interfaces lassen sich auf unterster Byte-Ebene
 * serialisieren.<br>
 * Diese Klasse aehnelt {@link Serializable}, jedoch muss hier
 * die Serialisierung explizit ausprogrammiert werden.
 * @author (c) 2012/2021 - Carsten Moeller - info@osm2po
 * @param <T> sollte im Normalfall die implementierende Klasse selbst sein
 *
 * @see #writeToStream(OutStream)
 * @see #readFromStream(InStream)
 * @see VarTypeDesk#registerType(byte, Class)
 * @see VarBean
 * @see VarNum
 * @see VarString
 */
public interface Var<T extends Var<? extends T>> {
    /**
     * Serialisiert dieses Objekt in einen Stream.
     * @param outStream {@link OutStream}
     * @see VarBean#writeToStream(OutStream)
     */
    void writeToStream(OutStream outStream);

    /**
     * Deserialisiert dieses Objekt aus einem Stream.
     * @param inStream {@link InStream}
     * @return <b>Muss</b> <code>this</code> zurueckgeben.
     * @see VarBean#readFromStream(InStream)
     */
    T readFromStream(InStream inStream);

}
