/*
 * $Id: VarBean.java 5482 2021-12-12 10:00:35Z carsten $
 *
 * Copyright 2017 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;

import static de.cm.osm2po.primitives.VarTypeDesk.TYPE_NULL;
import static de.cm.osm2po.primitives.VarTypeDesk.newInstanceOf;
import static de.cm.osm2po.primitives.VarTypeDesk.typeIdOf;

import java.io.File;

import de.cm.osm2po.Config;

/**
 * Kapselt ein {@link Var} inklusive seines Typs aus {@link VarTypeDesk}.
 * Bei der Serialisierung mittels {@link Var#writeToStream(OutStream)}
 * wird dieses Type-Byte {@link VarTypeDesk#typeIdOf(Var)} vorangestellt.
 * @author (c) 2017 - Carsten Moeller, info@osm2po.de
 * @see Config#getBean(String)
 * @see Config#loadBeans(String)
 */
public class VarBean implements Var<VarBean> {
    private byte varType;
    private Var<?> var;

    public VarBean() {}

    public VarBean(Var<?> var) {
        if (var != null) {
            this.varType = typeIdOf(var);
            this.var = var;
        } else {
            this.varType = TYPE_NULL;
        }
    }

    public void clear() {
        if (this.var != null) {
            this.var = null;
            this.varType = TYPE_NULL;
        }
    }

    public Var<?> get() {return this.var;}

    public final VarBean writeToFile(File file) {
        OutStream outStream = new OutStreamDisk(file);
        this.writeToStream(outStream);
        outStream.close();
        return this;
    }

    public final VarBean readFromFile(File file) {
        this.clear();
        InStream inStream = new InStreamDisk(file);
        this.readFromStream(inStream);
        inStream.close();
        return this;
    }


    @Override
    public void writeToStream(OutStream outStream) {
        outStream.writeByte(this.varType);
        if (this.varType != TYPE_NULL) {
            this.var.writeToStream(outStream);
        }
    }

    @Override
    public VarBean readFromStream(InStream inStream) {
        this.varType = inStream.readByte();
        if (this.varType != TYPE_NULL) {
            this.var = newInstanceOf(this.varType);
            this.var.readFromStream(inStream);
        }
        return this;
    }

    @Override
    public String toString() {
        return "[" + this.varType + "] "
            + (0 == this.varType ? "null" : this.var.toString());
    }

}
