/*
 * $Id: VarBooleanArray.java 4605 2018-01-09 20:31:08Z carsten $
 *
 * Copyright 2011 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;

import java.util.Arrays;

/**
 * Speicherfreundliches VarBooleanArray auf Bit-Ebene.
 * @author (c) 2013 - Carsten Moeller - info@osm2po
 */
public class VarBooleanArray implements Var<VarBooleanArray> {
    private int[] bits = new int[0];
    private int capacity = 0;

    public VarBooleanArray() {};

    public VarBooleanArray(int capacity) {
        this.bits = new int[((capacity-1) >>> 5) +1];
        this.capacity = capacity;
    }

    public int getCapacity() {return this.capacity;}

    public final void set(int idx, boolean bit) {
        if (idx < 0 || idx >= this.capacity)
            throw new ArrayIndexOutOfBoundsException(idx);

        int arrIdx = idx >>> 5;
        int bitIdx = idx - (arrIdx << 5);
        int bitVal = (1 << bitIdx);

        int intVal = this.bits[arrIdx];
        if (bit) intVal |= bitVal;
        else intVal &=~bitVal;

        this.bits[arrIdx] = intVal;
    }

    public final boolean get(int idx) {
        if (idx < 0 || idx >= this.capacity)
            throw new ArrayIndexOutOfBoundsException(idx);

        int arrIdx = idx >>> 5;
        int bitIdx = idx - (arrIdx << 5);
        int bitVal = 1 << bitIdx;

        boolean bit = (this.bits[arrIdx] & bitVal) != 0;
        return bit;
    }

    public VarBooleanArray copy() {
        VarBooleanArray ba = new VarBooleanArray();
        ba.bits = Arrays.copyOf(this.bits, this.bits.length);
        return ba;
    }

    @Override
    public void writeToStream(OutStream outStream) {
        outStream.writeInt(this.capacity);
        new VarIntArray(this.bits).writeToStream(outStream);
    }

    @Override
    public VarBooleanArray readFromStream(InStream inStream) {
        this.capacity = inStream.readInt();
        this.bits = new VarIntArray().readFromStream(inStream).getInts();
        return this;
    }

}
