/*
 * $Id: VarBooleanArray2D.java 4605 2018-01-09 20:31:08Z carsten $
 *
 * Copyright 2011 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;


/**
 * Speicherfreundliches, 2-dimensionales VarBooleanArray auf Bit-Ebene.
 * Benutzt inter {@link VarBooleanArray}
 * @author (c) 2016 - Carsten Moeller - info@osm2po
 */
public class VarBooleanArray2D implements Var<VarBooleanArray2D> {
    // TODO Arr[BoolenArray] wuerde MaxInt-Limit aufheben. Dann noch speicherfreundlich?
    private VarBooleanArray arr1D = new VarBooleanArray();
    private int capY = 0; // Capacity Y - Rows
    private int capX = 0; // Capacity X - Columns

    public VarBooleanArray2D() {}

    public VarBooleanArray2D(int capY, int capX) {
        if (capY < 1 || capX < 1 || ((long)capX * (long)capY) > Integer.MAX_VALUE)
            throw new IllegalArgumentException(
                    "Positive y and x expected where y*x must not exceed Int.MAX");
        this.capX = capX;
        this.capY = capY;
        this.arr1D = new VarBooleanArray(capY * capX);
    }

    public int getCapacityY() {return this.capY;}
    public int getCapacityX() {return this.capX;}

    private void checkRange(int y, int x) {
        if (y < 0 || x < 0 || y >= this.capY || x >= this.capX)
            throw new ArrayIndexOutOfBoundsException("[" + y + "][" + x + "]");
    }

    public final void set(int y, int x, boolean bit) {
        this.checkRange(y, x);
        int idx = y * this.capX + x;
        this.arr1D.set(idx, bit);
    }

    public final boolean get(int y, int x) {
        this.checkRange(y, x);
        int idx = y * this.capX + x;
        return this.arr1D.get(idx);
    }

    public VarBooleanArray2D copy() {
        VarBooleanArray2D ba2 = new VarBooleanArray2D(this.capY, this.capX);
        ba2.arr1D = this.arr1D.copy();
        return ba2;
    }

    @Override
    public void writeToStream(OutStream outStream) {
        outStream.writeInt(this.capY);
        outStream.writeInt(this.capX);
        this.arr1D.writeToStream(outStream);
    }

    @Override
    public VarBooleanArray2D readFromStream(InStream inStream) {
        this.capY = inStream.readInt();
        this.capX = inStream.readInt();
        this.arr1D = new VarBooleanArray().readFromStream(inStream);
        return this;
    }

}
