/*
 * $Id: VarByte.java 5243 2021-02-09 21:03:31Z carsten $
 *
 * Copyright 2011 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;

/**
 * Diese Klasse kapselt ein <code>byte</code> und ist aufgrund seiner
 * {@link Var} Ableitung serialisierbar.
 *
 * @author (c) 2012 - Carsten Moeller - info@osm2po
 */
public class VarByte implements Var<VarByte>, Comparable<VarByte> {

    private byte byteValue;

    public VarByte() {
        this.byteValue = 0;
    }

    public VarByte(byte byteValue) {
        this.byteValue = byteValue;
    }

    public byte getByte() {
        return this.byteValue;
    }

    public void setByte(byte byteValue) {
        this.byteValue = byteValue;
    }

    @Override
    public String toString() {
        return String.valueOf(this.byteValue);
    }

    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof VarByte)) return false;
        VarByte that = (VarByte) obj;
        return this.byteValue == that.byteValue;
    }

    @Override
    public int hashCode() {
        return this.byteValue;
    }

    @Override
    public VarByte readFromStream(InStream inStream) {
        this.byteValue = inStream.readByte();
        return this;
    }

    @Override
    public void writeToStream(OutStream outStream) {
        outStream.writeByte(this.byteValue);
    }

    @Override
    public int compareTo(VarByte that) {
        if (this.byteValue > that.byteValue) return 1;
        if (this.byteValue < that.byteValue) return -1;
        return 0;
    }

}
