/*
 * $Id: VarByteArray.java 3966 2015-11-16 19:15:40Z carsten $
 *
 * Copyright 2010 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;

/**
 * Ein Standard-Java-ByteArray fuer die Verwendung als {@link Var}.
 * @author (c) 2015 - Carsten Moeller - info@osm2po
 */
public class VarByteArray implements Var<VarByteArray> {
    private byte[] bytes;
    
    public VarByteArray() {}
    
    /**
     * CTOR. 
     * @param bytes Array of byte
     */
    public VarByteArray(byte[] bytes) {
    	if (null == bytes) {
    		this.bytes = new byte[0];
    	} else {
    		this.bytes = bytes; 
    	}
    }
    
    public byte[] getBytes() {
    	return this.bytes;
    }

    @Override
    public VarByteArray readFromStream(InStream inStream) {
        int n = inStream.readInt();
        this.bytes = new byte[n];
        for (int i = 0; i < n; i++) {
            this.bytes[i] = inStream.readByte();
        }
        return this;
    }

    @Override
    public void writeToStream(OutStream outStream) {
        int n = null == this.bytes ? 0 : this.bytes.length;
        outStream.writeInt(n);
        for (int i = 0; i < n; i++) {
            outStream.writeByte(this.bytes[i]);
        }
    }
    
    @Override
    public String toString() {
        String s = "[";
        int n = null == this.bytes ? 0 : this.bytes.length;
        if (n > 10) n = 10;
        for (int i = 0; i < n; i++) s += this.bytes[i] + ",";
        if (n > 0) s += "...";
        s += "]";
        return s;
    }

}
