package de.cm.osm2po.primitives;

import java.util.Arrays;

public class VarByteArray2D implements Var<VarByteArray2D> {

	private byte[][] arr;
	private int sizeY;
	private int sizeX;

	public VarByteArray2D() {
		this(0, 0);
	}

	public VarByteArray2D(int sizeY, int sizeX) {
		this.arr = new byte[sizeY][sizeX];
		this.sizeY = sizeY;
		this.sizeX = sizeX;
	}

	public final void clear() {
	    this.sizeX = 0;
	    this.sizeY = 0;
	    this.arr = null;
	}

	public final int getSizeY() {return this.sizeY;}
	public final int getSizeX() {return this.sizeX;}

	private void checkParams(int y, int x) {
		if (y < 0 || y >= this.sizeY)
			throw new IllegalArgumentException(
					"y must be in the range of 0 to " + (this.sizeY -1));
		if (x < 0 || x >= this.sizeX)
			throw new IllegalArgumentException(
					"x must be in the range of 0 to " + (this.sizeX -1));
	}

	public final void set(int y, int x, byte value) {
		this.checkParams(y, x);
		this.arr[y][x] = value;
	}

	public final byte get(int y, int x) {
		this.checkParams(y, x);
		return this.arr[y][x];
	}

	public final void fill(byte value) {
        for (int y = 0; y < this.sizeY; y++) {
            for (int x = 0; x < this.sizeX; x++) {
                this.arr[y][x] = value;
            }
        }
	}

    public final void xor(byte value) {
        for (int y = 0; y < this.sizeY; y++) {
            for (int x = 0; x < this.sizeX; x++) {
                this.arr[y][x] ^= value;
            }
        }
    }

	@Override
	public void writeToStream(OutStream outStream) {
		outStream.writeInt(this.sizeY);
		outStream.writeInt(this.sizeX);
		for (int y = 0; y < this.sizeY; y++) {
			for (int x = 0; x < this.sizeX; x++) {
				outStream.writeByte(this.arr[y][x]);
			}
		}
	}

	@Override
	public VarByteArray2D readFromStream(InStream inStream) {
		this.sizeY = inStream.readInt();
		this.sizeX = inStream.readInt();
		this.arr = new byte[this.sizeY][this.sizeX];
		for (int y = 0; y < this.sizeY; y++) {
			for (int x = 0; x < this.sizeX; x++) {
				this.arr[y][x] = inStream.readByte();
			}
		}
		return this;
	}

	@Override
	public boolean equals(Object obj) {
		if (null == obj) return false;
		if (!(obj instanceof VarByteArray2D)) return false;
		if (this == obj) return true;
		VarByteArray2D that = (VarByteArray2D) obj;
		if (this.sizeY != that.sizeY) return false;
		if (this.sizeX != that.sizeX) return false;
		for (int y = 0; y < this.sizeY; y++) {
			for (int x = 0; x < this.sizeX; x++) {
				if (this.arr[y][x] != that.arr[y][x]) return false;
			}
		}
		return true;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + Arrays.deepHashCode(this.arr);
		result = prime * result + this.sizeX;
		result = prime * result + this.sizeY;
		return result;
	}

	public final String debugToString() {
	    String s = "";
	    for (int y = sizeY -1; y >= 0; y--) {
	        if (y < sizeY -1) s += '\n';
	        for (int x = 0; x < sizeX; x++) {
	            if (x > 0) s += ' ';
	            String hex = Integer.toBinaryString(get(y, x));
	            if (hex.length() == 1) hex = '0' + hex;
	            s += hex;
	        }
	    }
	    return s;
	}

}
