/*
 * $Id: VarInt.java 5243 2021-02-09 21:03:31Z carsten $
 *
 * Copyright 2011 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;

/**
 * Diese Klasse kapselt einen <code>int</code> und ist aufgrund seiner
 * {@link Var} Ableitung serialisierbar.
 *
 * @author (c) 2012 - Carsten Moeller - info@osm2po
 */
public class VarInt implements Var<VarInt>, Comparable<VarInt> {

    private int intValue;

    public VarInt() {
        this.intValue = 0;
    }

    public VarInt(int intValue) {
        this.intValue = intValue;
    }

    public int getInt() {
        return this.intValue;
    }

    public void setInt(int intValue) {
        this.intValue = intValue;
    }

    @Override
    public String toString() {
        return String.valueOf(this.intValue);
    }

    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof VarInt)) return false;
        VarInt that = (VarInt) obj;
        return this.intValue == that.intValue;
    }

    @Override
    public int hashCode() {
        return this.intValue;
    }

    @Override
    public VarInt readFromStream(InStream inStream) {
        this.intValue = inStream.readInt();
        return this;
    }

    @Override
    public void writeToStream(OutStream outStream) {
        outStream.writeInt(this.intValue);
    }

    @Override
    public int compareTo(VarInt that) {
        if (this.intValue > that.intValue) return 1;
        else if (this.intValue < that.intValue) return -1;
        return 0;
    }

}
