/*
 * $Id: VarIntArray.java 5243 2021-02-09 21:03:31Z carsten $
 *
 * Copyright 2010 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;
import java.util.Arrays;

/**
 * Ein Standard-Java-IntArray fuer die Verwendung als {@link Var}.
 * @author (c) 2015 - Carsten Moeller - info@osm2po
 */
public class VarIntArray implements Var<VarIntArray> {

    private int[] ints;

    public VarIntArray() {}

    /**
     * CTOR.
     * @param ints Array of int
     */
    public VarIntArray(int[] ints) {
    	if (null == ints) {
    		this.ints = new int[0];
    	} else {
    		this.ints = ints;
    	}
    }

    public int[] getInts() {
    	return this.ints;
    }

    @Override
    public VarIntArray readFromStream(InStream inStream) {
        int n = inStream.readInt();
        this.ints = new int[n];
        for (int i = 0; i < n; i++) {
            this.ints[i] = inStream.readInt();
        }
        return this;
    }

    @Override
    public void writeToStream(OutStream outStream) {
        int n = null == this.ints ? 0 : this.ints.length;
        outStream.writeInt(n);
        for (int i = 0; i < n; i++) {
            outStream.writeInt(this.ints[i]);
        }
    }

    @Override
    public String toString() {
        return Arrays.toString(this.ints);
    }

}
