/*
 * $Id: VarIntInt.java 5243 2021-02-09 21:03:31Z carsten $
 *
 * Copyright 2010 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;

/**
 * Bildet eine Key-Value-Klammer um einen int als Schluessel
 * und einen weiteren als Wert.
 * @author (c) 2021 - Carsten Moeller - info@osm2po.de
 */
public class VarIntInt
implements Var<VarIntInt>, Comparable<VarIntInt> { 
    
    private int key;
    private int val;

    public VarIntInt() {}
    
    public VarIntInt(int key, int val) {
        this.key = key;
        this.val = val;
    }
    
    public int getKey() {return this.key;}
    public int getVal() {return this.val;}
    
    @Override
    public VarIntInt readFromStream(InStream inStream) {
        this.key = inStream.readInt();
        this.val = inStream.readInt();
        return this;
    }
    
    @Override
    public void writeToStream(OutStream outStream) {
        outStream.writeInt(this.key);
        outStream.writeInt(this.val);
    }

    @Override
    public int compareTo(VarIntInt that) {
        if (this.key > that.key) return 1;
        else if (this.key < that.key) return -1;
        return 0;
    }

}
