/*
 * $Id: VarIntInts.java 5243 2021-02-09 21:03:31Z carsten $
 *
 * Copyright 2010 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;

/**
 * Bildet eine Key-Value-Klammer um einen int als Schluessel
 * und ein Array von ints als Wert.
 * @author (c) 2021 - Carsten Moeller - info@osm2po.de
 */
public class VarIntInts
implements Var<VarIntInts>, Comparable<VarIntInts> {

    private int key;
    private int[] val;

    public int getKey() {return key;}
    public int[] getVal() {return val;}

    public VarIntInts() {
        key = 0;
        val = new int[0];
    }

    public VarIntInts(int key, int[] val) {
        this.key = key;
        this.val = val;
    }

    @Override
    public int compareTo(VarIntInts that) {
        if (this.key > that.key) return 1;
        else if (this.key < that.key) return -1;
        return 0;
    }

    @Override
    public void writeToStream(OutStream outStream) {
        outStream.writeInt(this.key);
        int len = this.val.length;
        VarNum.writeUnsigned(len, outStream);
        for (int i = 0; i < len; i++) outStream.writeInt(this.val[i]);
    }

    @Override
    public VarIntInts readFromStream(InStream inStream) {
        this.key =  inStream.readInt();
        int len = (int) VarNum.readUnsigned(inStream);
        this.val = new int[len];
        for (int i = 0; i < len; i++) this.val[i] = inStream.readInt();
        return this;
    }

}
