/*
 * $Id: VarLong.java 5243 2021-02-09 21:03:31Z carsten $
 *
 * Copyright 2011 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;

/**
 * Diese Klasse kapselt einen <code>long</code> und ist aufgrund seiner
 * {@link Var} Ableitung serialisierbar.
 *
 * @author (c) 2015 - Carsten Moeller - info@osm2po
 */
public class VarLong implements Var<VarLong>, Comparable<VarLong> {

    private long longValue;

    public VarLong() {
        this.longValue = 0L;
    }

    public VarLong(long longValue) {
        this.longValue = longValue;
    }

    public long getLong() {
        return this.longValue;
    }

    public void setLong(long longValue) {
        this.longValue = longValue;
    }

    @Override
    public String toString() {
        return String.valueOf(this.longValue);
    }

    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof VarLong)) return false;
        VarLong that = (VarLong) obj;
        return this.longValue == that.longValue;
    }

    @Override
    public int hashCode() {
        // HashCode wie gross Long
        return (int)(this.longValue ^ (this.longValue >>> 32));
    }

    @Override
    public VarLong readFromStream(InStream inStream) {
        this.longValue = inStream.readLong();
        return this;
    }

    @Override
    public void writeToStream(OutStream outStream) {
        outStream.writeLong(this.longValue);
    }

    @Override
    public int compareTo(VarLong that) {
        if (this.longValue > that.longValue) return 1;
        if (this.longValue < that.longValue) return -1;
        return 0;
    }

}
