/*
 * $Id: VarShort.java 5243 2021-02-09 21:03:31Z carsten $
 *
 * Copyright 2017 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;

/**
 * Diese Klasse kapselt ein <code>short</code> und ist aufgrund seiner
 * {@link Var} Ableitung serialisierbar.
 *
 * @author (c) 2017 - Carsten Moeller - info@osm2po
 */
public class VarShort implements Var<VarShort>, Comparable<VarShort> {

    private short shortValue;

    public VarShort() {
        this.shortValue = 0;
    }

    public VarShort(short shortValue) {
        this.shortValue = shortValue;
    }

    public short getShort() {
        return this.shortValue;
    }

    public void setShort(short shortValue) {
        this.shortValue = shortValue;
    }

    @Override
    public String toString() {
        return String.valueOf(this.shortValue);
    }

    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof VarShort)) return false;
        VarShort that = (VarShort) obj;
        return this.shortValue == that.shortValue;
    }

    @Override
    public int hashCode() {
        return this.shortValue;
    }

    @Override
    public VarShort readFromStream(InStream inStream) {
        this.shortValue = inStream.readShort();
        return this;
    }

    @Override
    public void writeToStream(OutStream outStream) {
        outStream.writeShort(this.shortValue);
    }

    @Override
    public int compareTo(VarShort that) {
        if (this.shortValue > that.shortValue) return 1;
        if (this.shortValue < that.shortValue) return -1;
        return 0;
    }

}
