/*
 * $Id: VarStringInt.java 5245 2021-02-12 19:52:51Z carsten $
 *
 * Copyright 2010 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;

import static de.cm.osm2po.misc.Utils.UTF8;

/**
 * Bildet eine Key-Value-Klammer um einen {@link String} als Schluessel
 * und einen int als Wert.
 * @author (c) 2021 - Carsten Moeller - info@osm2po.de
 */
public class VarStringInt
implements Var<VarStringInt>, Comparable<VarStringInt> {

    private String key;
    private int val;

    public String getKey() {return key;}
    public int getVal() {return val;}

    public VarStringInt() {
        key = "";
        val = 0;
    }

    public VarStringInt(String key) {
        this.key = key;
        this.val = 0;
    }

    public VarStringInt(String key, int val) {
        this.key = key;
        this.val = val;
    }

    @Override
    public int compareTo(VarStringInt that) {
        return this.key.compareTo(that.key);
    }

    @Override
    public void writeToStream(OutStream outStream) {
        byte[] keyBytes = this.key.getBytes(UTF8);
        int len = keyBytes.length;
        VarNum.writeUnsigned(len, outStream);
        for (int i = 0; i < len; i++) outStream.writeByte(keyBytes[i]);
        outStream.writeInt(this.val);
    }

    @Override
    public VarStringInt readFromStream(InStream inStream) {
        int len = (int) VarNum.readUnsigned(inStream);
        byte[] keyBytes = new byte[len];
        for (int i = 0; i < len; i++) keyBytes[i] = inStream.readByte();
        this.key = new String(keyBytes, UTF8);
        this.val = inStream.readInt();
        return this;
    }

}
