/*
 * $Id: VarStringInts.java 5243 2021-02-09 21:03:31Z carsten $
 *
 * Copyright 2010 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.primitives;

import static de.cm.osm2po.misc.Utils.UTF8;

/**
 * Bildet eine Key-Value-Klammer um einen {@link String} als Schluessel
 * und ein Array von ints als Wert.
 * @author (c) 2021 - Carsten Moeller - info@osm2po.de
 */
public class VarStringInts
implements Var<VarStringInts>, Comparable<VarStringInts> {

    private String key;
    private int[] val;

    public String getKey() {return key;}
    public int[] getVal() {return val;}

    public VarStringInts() {
        key = "";
        val = new int[0];
    }

    public VarStringInts(String key) {
        this.key = key;
        this.val = new int[0];
    }

    public VarStringInts(String key, int[] val) {
        this.key = key;
        this.val = val;
    }

    @Override
    public int compareTo(VarStringInts that) {
        return this.key.compareTo(that.key);
    }

    @Override
    public void writeToStream(OutStream outStream) {
        {
            byte[] keyBytes = this.key.getBytes(UTF8);
            int len = keyBytes.length;
            VarNum.writeUnsigned(len, outStream);
            for (int i = 0; i < len; i++) outStream.writeByte(keyBytes[i]);
        } {
            int len = this.val.length;
            outStream.writeInt(len);
            for (int i = 0; i < len; i++) outStream.writeInt(this.val[i]);
        }
    }

    @Override
    public VarStringInts readFromStream(InStream inStream) {
        {
            int len = (int) VarNum.readUnsigned(inStream);
            byte[] keyBytes = new byte[len];
            for (int i = 0; i < len; i++) keyBytes[i] = inStream.readByte();
            this.key = new String(keyBytes, UTF8);
        } {
            int len = inStream.readInt();
            this.val = new int[len];
            for (int i = 0; i < len; i++) this.val[i] = inStream.readInt();
        }
        return this;
    }

}
