/*
 * $Id: RouterConstraints.java 5420 2021-11-16 20:31:42Z carsten $
 *
 * Copyright 2021 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.routing;

/**
 * Abbruchkriterium fuer die Traversierung eines Graphen.<br>
 * Diese ist entweder das Erreichen der ersten oder aller targetIds oder
 * die Ueberschreitung der maxCost. Wird keine targetId angegeben, so wird
 * dennoch intern die erste Id im Array auf den Wert 0 gesetzt. 0 bewirkt,
 * dass der komplette Graph traversiert wird, da 0 nicht erreichbar ist.
 * @author (c) 2021 - Carsten Moeller - info@osm2po
 * @see AbstractRouter#traverse(int, RouterConstraints, RouterParameters)
 */
public class RouterConstraints {
    public final double maxCost;
    public final int[] targetIds;

    public RouterConstraints(double maxCost, int... targetId) {
        this.maxCost = maxCost <= 0 ? Double.MAX_VALUE : maxCost;
        this.targetIds = (null == targetId || 0 == targetId.length) ? new int[] {0} : targetId;
    }

    public RouterConstraints(int... targetId) {
        this(Double.MAX_VALUE, targetId);
    }
}