/*
 * $Id: RouterParameters.java 5420 2021-11-16 20:31:42Z carsten $
 *
 * Copyright 2021 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.routing;

import java.util.Properties;

/**
 * @author (c) 2021 - Carsten Moeller - info@osm2po
 */
public class RouterParameters {
    private Properties props;
    private boolean findShortestPath;
    private boolean ignoreRestrictions;
    private boolean ignoreOneWays;
    private boolean resetOnZero;
    private boolean findReverse;
    private double heuristicFactor; // Bewertungsfaktor 0 Dijkstra, 1 AStar
    private double localityFactor; // Bewertungsfaktor fuer MultiLevel

    public RouterParameters() {this(null);}
    public RouterParameters(Properties params) {
        if (null == params) params = new Properties();
        this.props = params;

        this.findShortestPath = Boolean.valueOf(
                params.getProperty("findShortestPath", "false"));
        this.findReverse = Boolean.valueOf(
                params.getProperty("findReverse", "false"));
        this.ignoreRestrictions = Boolean.valueOf(
                params.getProperty("ignoreRestrictions", "false"));
        this.ignoreOneWays = Boolean.valueOf(
                params.getProperty("ignoreOneWays", "false"));
        this.resetOnZero = Boolean.valueOf(
                params.getProperty("resetOnZero", "false"));
        this.heuristicFactor = Double.parseDouble(
                params.getProperty("heuristicFactor", "0")); // 1 = A*
        this.localityFactor = Double.parseDouble(
                params.getProperty("localityFactor", "0")); // 0 = off
    }

    /**
     * Liefert den Rohwert aus den internen {@link Properties}.
     * @param key String key
     * @param defaultValue String Wert wenn key nicht gefunden wurde.
     * @return interne {@link Properties}
     */
    public final String getProperty(final String key, final String defaultValue) {
        return this.props.getProperty(key, defaultValue);
    }

    /** @return true: Kuerzeste Strecke, sonst schnellste, default: false */
    public final boolean isFindShortestPath() {
        return this.findShortestPath;
    }
    /** @return true: Abbiegevorschriften ignorieren, default: false */
    public final boolean isIgnoreRestrictions() {
        return this.ignoreRestrictions;
    }
    /** @return true: Einbahnstrassen ignorieren, default: false */
    public final boolean isIgnoreOneWays() {
        return this.ignoreOneWays;
    }
    /** @return true: Bei Trigger PQ-Kosten auf Null setzen, default: false */
    public final boolean isResetOnZero() {
        return this.resetOnZero;
    }
    /** @return true: Rueckwaerts routen, default: false */
    public final boolean isFindReverse() {
        return this.findReverse;
    }
    /** @return 0: Dijkstra, 1: AStar, sonst experimentell, default: 0 */
    public final double getHeuristicFactor() {
        return this.heuristicFactor;
    }
    /** @return 0:Aus, sonst Vergroesserungsfaktor fuer Zielgebiet beim MultiLevelGrids, default: 0 */
    public final double getLocalityFactor() {
        return this.localityFactor;
    }


    public final RouterParameters setFindShortestPath(boolean findShortestPath) {
        this.findShortestPath = findShortestPath;
        return this;
    }
    public final RouterParameters setResetOnZero(boolean resetOnZero) {
        this.resetOnZero = resetOnZero;
        return this;
    }
    public final RouterParameters setFindReverse(boolean findReverse) {
        this.findReverse = findReverse;
        return this;
    }
    public final RouterParameters setHeuristicFactor(double heuristicFactor) {
        this.heuristicFactor = heuristicFactor;
        return this;
    }
    public final RouterParameters setLocalityFactor(double localityFactor) {
        this.localityFactor = localityFactor;
        return this;
    }
    public final RouterParameters setIgnoreRestrictions(boolean ignoreRestrictions) {
        this.ignoreRestrictions = ignoreRestrictions;
        return this;
    }
    public final RouterParameters setIgnoreOneWays(boolean ignoreOneWays) {
        this.ignoreOneWays = ignoreOneWays;
        return this;
    }

}