/*
 * $Id: RoutingResultSegment.java 5123 2020-10-19 19:43:39Z carsten $
 *
 * Copyright 2011 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.routing;

import de.cm.osm2po.model.LatLon;
import de.cm.osm2po.model.LatLons;
import de.cm.osm2po.primitives.InStream;
import de.cm.osm2po.primitives.OutStream;
import de.cm.osm2po.primitives.Var;
import de.cm.osm2po.primitives.VarString;

/**
 * Kapselt das Segment-Resultat nach einem Routing.
 * @author (c) 2012 - Carsten Moeller - info@osm2po
 */
public final class RoutingResultSegment implements Var<RoutingResultSegment> {

    private int edgeId;
    private int sourceId;
    private int targetId;
    private float h;
    private float km;
    private int flags;
    private byte noTurnBits;
    private LatLons coords;
    private VarString name;

    public RoutingResultSegment() {
        this.coords = new LatLons();
        this.name = new VarString("");
    }
    
    public void setEdgeId(int edgeId) {this.edgeId = edgeId;}
    public LatLons getCoords() {return this.coords;}
    public void setCoords(LatLons coords) {this.coords = coords;}
    public VarString getName() {return this.name;}
    public void setName(VarString name) {this.name = name;}

    public String getStreetName() {return this.name.toString();}

    public LatLon[] getLatLons() {
        if (null == this.coords) return null;
        int nCoords = this.coords.getSize();
        LatLon[] latLons = new LatLon[nCoords];
        for (int i = 0; i < nCoords; i++) {
            latLons[i] = new LatLon();
            latLons[i].setCoord(this.coords.getCoord(i));
        }
        return latLons;
    }
    
    public int getSourceId() {return this.sourceId;}
    public void setSourceId(int sourceId) {this.sourceId = sourceId;}
    public int getTargetId() {return this.targetId;}
    public void setTargetId(int targetId) {this.targetId = targetId;}
    public float getH() {return this.h;}
    public void setH(float h) {this.h = h;}
    public float getKm() {return this.km;}
    public void setKm(float km) {this.km = km;}
    public int getFlags() {return this.flags;}
    public void setFlags(int flags) {this.flags = flags;}
    public byte getNoTurnBits() {return this.noTurnBits;}
    public void setNoTurnBits(byte noTurnBits) {this.noTurnBits = noTurnBits;}
    
    public int getId() {return Graph.toSegmentId(this.edgeId);}
    public boolean isWrongWay() {return Graph.isWrongWay(this.edgeId);}
    public boolean isOneWay() {return Graph.isOneWay(this.edgeId);}
    public boolean isReverse() {return Graph.isReverse(this.edgeId);}
    
    @Override
    public String toString() {
        String s = "Id:" + this.getId()
        + " Source:" + this.sourceId
        + " Target:" + this.targetId
        + " Flags:" + this.flags
        + " NoTurnBits:" + this.noTurnBits
        + " OneWay:" + this.isOneWay()
        + " WrongWay:" + this.isWrongWay()
        + " Reverse:" + this.isReverse()
        + " h:" + this.h
        + " Km:" + this.km
        + " Name:" + this.name
        + " Geometry:"  + this.coords;

        return s;
    }

    @Override
    public RoutingResultSegment readFromStream(InStream inStream) {
        this.edgeId = inStream.readInt();
        this.sourceId = inStream.readInt();
        this.targetId = inStream.readInt();
        this.flags = inStream.readInt();
        this.noTurnBits = inStream.readByte();
        this.h = Float.intBitsToFloat(inStream.readInt());
        this.km = Float.intBitsToFloat(inStream.readInt());
        this.name.readFromStream(inStream);
        this.coords.readFromStream(inStream);
        return this;
    }

    @Override
    public void writeToStream(OutStream outStream) {
        outStream.writeInt(this.edgeId);
        outStream.writeInt(this.sourceId);
        outStream.writeInt(this.targetId);
        outStream.writeInt(this.flags);
        outStream.writeByte(this.noTurnBits);
        outStream.writeInt(Float.floatToIntBits(this.h));
        outStream.writeInt(Float.floatToIntBits(this.km));
        this.name.writeToStream(outStream);
        this.coords.writeToStream(outStream);
    }

}
