/*
 * $Id: ServiceFilter.java 4974 2018-10-25 10:42:01Z carsten $
 *
 * Copyright 2017 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.service;

import com.sun.net.httpserver.HttpExchange;

import de.cm.osm2po.Config;
import de.cm.osm2po.service.errors.ServiceException;
import de.cm.osm2po.service.filters.CorsFilter;
import de.cm.osm2po.service.filters.HttpLogFilter;
import de.cm.osm2po.service.filters.RateLimitFilter;

/**
 * Wird vor der Verarbeitung und nach dem Senden
 * der Response in {@link Osm2poService#handle(HttpExchange)}
 * gerufen.
 * @author (c) 2017 - Carsten Moeller - info@osm2po
 * @see #preHandle(ServiceRequest)
 * @see RateLimitFilter
 * @see HttpLogFilter
 * @see CorsFilter
 */
public interface ServiceFilter {

    /**
     * Quasi-Konstruktor wird von {@link Osm2poService} gerufen.
     * @param config {@link Config}
     * @param index int Index in der Konfiguration
     */
    void init(Config config, int index);
    /** Signal, dass Server gleich runterfaehrt. */
    void shutDown();

    /**
     * Filtert einen HTTP-Request vor der Verarbeitung.
     * @param serviceRequest {@link ServiceRequest}
     * @return false: Durchlassen, true: Aufrufer muss return ausloesen
     * @throws Exception z.B. {@link ServiceException}
     */
    boolean preHandle(ServiceRequest serviceRequest) throws Exception;

    /**
     * Wird nach dem Schreiben der Response gerufen.
     * Jedoch nur, wenn dieser Filter auch ein
     * {@link #preHandle(ServiceRequest)} erhalten hat.
     * @param serviceRequest {@link ServiceRequest}, NULLABLE
     * @param serviceResponse {@link ServiceResponse}, NULLABLE
     * @param sent true: Response gesendet, auch 404 und Co.
     */
    void postHandle(ServiceRequest serviceRequest,
            ServiceResponse serviceResponse, boolean sent);

}
