/*
 * $Id: ServiceHandler.java 5495 2021-12-29 19:55:22Z carsten $
 *
 * Copyright 2011 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.service;

import java.util.Properties;

import de.cm.osm2po.Config;
import de.cm.osm2po.routing.Graph;

/**
 * Behandelt Requests des {@link Osm2poService}.
 * @author (c) 2015 - Carsten Moeller - info@osm2po
 * @see #handle(String, Config, Graph, Properties, ServiceResourcesChecker)
 */
public interface ServiceHandler {

    public final static String MIME_TEXT = "text/plain; charset=utf-8";
    public final static String MIME_HTML = "text/html; charset=utf-8";
    public final static String MIME_JSON = "application/json; charset=utf-8";

    /** OK */
    public final static int HTTP_200 = 200;
    /** OK (accepted) - convention: show in MessageBox instead */
    public final static int HTTP_202 = 202;
    /** Not Found */
    public final static int HTTP_404 = 404;
    /** Unsupported Operation */
    public final static int HTTP_501 = 501;

    /**
     * Handler fuer {@link Osm2poService}.
     * @param cmd String beliebiger Kommando-Name.
     * @param config {@link Config}
     * @param graph {@link Graph}
     * @param request Query-Parameter als {@link Properties}
     * @param src {@link ServiceResourcesChecker}
     * @return {@link ServiceResponse} oder NULL fuer "Nicht verarbeitet".
     * @throws Exception beliebiger Natur
     */
    ServiceResponse handle(final String cmd, Config config, Graph graph,
            Properties request, ServiceResourcesChecker src)
    throws Exception;
}