/*
 * $Id: ServiceRequest.java 4749 2018-03-29 14:50:52Z carsten $
 *
 * Copyright 2017 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.service;

import static de.cm.osm2po.service.Osm2poService.getHeaderValue;
import static java.lang.System.currentTimeMillis;

import java.util.Properties;

import com.sun.net.httpserver.HttpExchange;

public final class ServiceRequest {

    public final long uid;
    public final String cmd;
    public final Properties params;
    public final HttpExchange htx;
    public final String ip;

    public ServiceRequest(String cmd, Properties params, HttpExchange htx) {
        this.cmd = cmd;
        this.params = params;
        this.htx = htx;
        this.uid = createUID();
        this.ip = resolveIp(htx);
    }

    private static short UIDCOUNTER = 0;
    private synchronized static long createUID() {
        final long D2000 = 60907590000000L; // 1.1.2000
        return currentTimeMillis() - D2000 + (UIDCOUNTER++);
    }

    private static String resolveIp(HttpExchange htx) {
        String ip = getHeaderValue(htx.getRequestHeaders(), "x-forwarded-for");
        if (null == ip) { // Kein Proxy dazwischen, dann echte IP
            ip = htx.getRemoteAddress().getAddress().getHostAddress(); // IP
        }
        return ip;
    }
}
