/*
 * $Id: ServiceResourcesChecker.java 5211 2020-11-04 20:28:21Z carsten $
 *
 * Copyright 2017 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.service;

/**
 * Sollte von einem Server implementiert werden, um
 * angeschlossenen Handlern die Moeglichkeit zu eroeffnen,
 * intensive Prozesse bei Resourcen-Mangel per Exception
 * zu unterbinden.
 * @author (c) 2017 - Carsten Moeller, info@osm2po.de
 * @see #checkResources()
 */
public interface ServiceResourcesChecker {
    /**
     * Wirft eine Exception, wenn zu viele intensive
     * Prozesse im Host, z.B {@link Osm2poService} laufen.<br>
     * Wird gewoehnlich in einem {@link ServiceHandler}
     * gerufen, bevor der Prozess weitere Resourcen verschlingt.
     * @see #resourcesReleased()
     * @throws Exception beliebiger Natur
     */
    void checkResources() throws Exception;

    /**
     * Kann optional aus einem Handler mit eigenem Exception-Handling
     * gerufen werden, um den Host ueber die Anstehende Beendigung des
     * Threads zu informieren, also darueber, dass Resourcen in Kuerze
     * wieder frei werden.
     */
    void resourcesReleased();
}
