/*
 * $Id: ErrorEnum.java 5480 2021-12-11 13:05:00Z carsten $
 *
 * Copyright 2017 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.service.errors;

import java.io.InputStream;
import java.text.MessageFormat;
import java.util.Properties;

public enum ErrorEnum {

    OK(0, 200),
    ERR_ROUTE_NOT_FOUND(1, 404),
    ERR_TSP_NOT_FOUND(2, 404),
    ERR_TOO_FEW_POINTS(3, 400),
    ERR_ROUTER_NOT_FOUND(10, 400),
    ERR_PARAMETER_INVALID(100, 400),
    ERR_SOURCE_TARGET_EQUAL(110, 400),
    ERR_NO_DATA_FOUND(200, 404),
    ERR_SOURCE_INVALID(111, 400),
    ERR_TARGET_INVALID(112, 400),
    ERR_SOURCE_NOT_FOUND(121, 404),
    ERR_TARGET_NOT_FOUND(122, 404),
    ERR_CLOSEST_VERTEX_NOT_FOUND(301, 404),
    ERR_CLOSEST_OVERLAYNODE_NOT_FOUND(302, 404),
    ERR_TOO_MANY_HEAVY_REQUESTS(401, 509),
    ERR_OUT_OF_MEMORY(402, 507),
    ERR_RATELIMIT_EXCEEDED(403, 429),
    ERR_HTTP_METHOD_NOT_ALLLOWED(404, 405),
    ERR_HTTP_METHOD_POST_BADDATA(405, 400),
    ERR_UNAUTHORIZED(405, 401),
    ERR_ACCOUNT_EXPIRED(406, 423),
    ERR_DOWN_MAINTENANCE(407, 503),
    ERR_UNEXPECTED_EXCEPTION(900, 500),
    ERR_UNEXPECTED_ILLEGAL_ARGUMENT(901, 400),
    ERR_UNEXPECTED_NUMBER_FORMAT(902, 400),
    ERR_UNEXPECTED_UNSUPPORTED_OPERATION(903, 501),
    ERR_UNEXPECTED_ILLEGAL_STATE(904, 500);

    private static Properties messages;
    static {
        try { // Lazy Init
            messages = new Properties();
            InputStream is = ServiceException.class.getResourceAsStream("errors.i18n");
            if (is != null) {
                messages.load(is);
                is.close();
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private static String getLocalizedMessage(LangEnum lang, int errNo, Object... args) {
        String key = "ERR." + errNo;
        if (null == lang) lang = LangEnum.Default;
        String m = messages.getProperty(key + "." + lang.getLang());
        if (null == m) m = messages.getProperty(key);
        if (null == m) m = "";
        if (args != null) m = MessageFormat.format(m, args);
        return m;
    }

    private int number;
    private int httpCode;

    private ErrorEnum(int number, int httpCode) {
        this.number = number;
        this.httpCode = httpCode;
    }

    public int getNumber() {
        return this.number;
    }

    public int getHttpCode() {
        return this.httpCode;
    }

    public String getMessage(LangEnum lang, Object...args) {
        return getLocalizedMessage(lang, this.number, args);
    }

    @Override
    public String toString() {
        return this.getNumber() + ": " + this.getMessage(null);
    }
}
