/*
 * $Id: CorsFilter.java 4974 2018-10-25 10:42:01Z carsten $
 *
 * Copyright 2017 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.service.filters;

import static de.cm.osm2po.service.Osm2poService.getHeaderValue;

import com.sun.net.httpserver.Headers;

import de.cm.osm2po.Config;
import de.cm.osm2po.service.ServiceFilter;
import de.cm.osm2po.service.ServiceRequest;
import de.cm.osm2po.service.ServiceResponse;

/**
 * Ein {@link ServiceFilter}, der CORS
 * (CrossOriginResourceSharing) erlaubt.
 * @author (c) 2017 - Carsten Moeller - info@osm2po
 */
public class CorsFilter implements ServiceFilter {

    @Override
    public void init(Config config, int index) {
        config.getLog().warn(this.getClass().getSimpleName() + " active!");
    }
    @Override public void shutDown() {}
    @Override public void postHandle(
            ServiceRequest req, ServiceResponse resp, boolean sent) {}

    @Override
    public boolean preHandle(ServiceRequest sr) throws Exception {
        Headers respH = sr.htx.getResponseHeaders();
        respH.set("Access-Control-Allow-Origin", "*");
        // respH.set("Access-Control-Max-Age", "600"); Wirkt nur bei gleichen Paramentern

        if ("OPTIONS".equals(sr.htx.getRequestMethod())) { // CORS PreFlight
            // https://remysharp.com/2011/04/21/getting-cors-working
            // http://restlet.com/company/blog/2015/12/15/understanding-and-using-cors/
            // https://hpbn.co/xmlhttprequest/

            Headers reqH = sr.htx.getRequestHeaders();
            String hval;
            hval = getHeaderValue(reqH, "Access-Control-Request-Headers");
            if (hval != null) respH.set("Access-Control-Allow-Headers", hval);
            hval = getHeaderValue(reqH, "Access-Control-Request-Method");
            if (hval != null) respH.set("Access-Control-Allow-Methods", hval);
            sr.htx.sendResponseHeaders(200, 0);
            sr.htx.close(); // Wichtig, sonst kein Response
            return true;
        }

        return false;
    }

}
