/*
 * $Id: PositionHandler.java 4566 2017-11-19 16:53:59Z carsten $
 *
 * Copyright 2011 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.service.handlers;

import static de.cm.osm2po.misc.Utils.roundE7;
import static de.cm.osm2po.routing.RoutingResultWriter.writeOverlayNodeAsCsv;
import static de.cm.osm2po.routing.RoutingResultWriter.writeOverlayNodeAsFeaturePoint;
import static de.cm.osm2po.routing.RoutingResultWriter.writeVertexAsCsv;
import static de.cm.osm2po.routing.RoutingResultWriter.writeVertexAsFeaturePoint;
import static de.cm.osm2po.service.ServiceHelpers.parseDoubleProperty;
import static de.cm.osm2po.service.errors.ErrorEnum.ERR_CLOSEST_VERTEX_NOT_FOUND;

import java.io.ByteArrayOutputStream;
import java.util.Properties;

import de.cm.osm2po.Config;
import de.cm.osm2po.routing.Graph;
import de.cm.osm2po.routing.OverlayNode;
import de.cm.osm2po.service.Osm2poService;
import de.cm.osm2po.service.ServiceHandler;
import de.cm.osm2po.service.ServiceResourcesChecker;
import de.cm.osm2po.service.ServiceResponse;
import de.cm.osm2po.service.errors.ServiceException;

/**
 * Behandelt Requests des {@link Osm2poService}.
 * @author (c) 2015 - Carsten Moeller - info@osm2po
 */
public class PositionHandler implements ServiceHandler {
    @Override
    public ServiceResponse handle(final String cmd, Config config, Graph graph,
            Properties request, ServiceResourcesChecker lttc) throws Exception {

        byte[] response = null;
        boolean isGeoJson = "geojson".equals((request.getProperty("format")));

        if ("fv".equals(cmd)) {
            ByteArrayOutputStream baos = new ByteArrayOutputStream(0x100);
            double lat = parseDoubleProperty(request, "lat", 0);
            double lon = parseDoubleProperty(request, "lon", 0);

            // even fetch if OL wrapDateLine=true
            lon = (lon + 3600 + 180) % 360 - 180;
            lat = (lat + 1800 + 90) % 180 - 90;

            int minId = graph.findClosestVertexId((float)lat, (float)lon);
            if (minId < 1) throw new ServiceException(ERR_CLOSEST_VERTEX_NOT_FOUND);
            lat = roundE7(graph.getLats()[minId]);
            lon = roundE7(graph.getLons()[minId]);
            if (isGeoJson) writeVertexAsFeaturePoint(baos, minId, lon, lat);
            else writeVertexAsCsv(baos, minId, lon, lat);
            response = baos.toByteArray();
            return new ServiceResponse(HTTP_200, isGeoJson ? MIME_JSON : MIME_TEXT, response);

        } else if ("fo".equals(cmd)) { // Find OverlayNode
            ByteArrayOutputStream baos = new ByteArrayOutputStream(0x100);
            double lat = parseDoubleProperty(request, "lat", 0);
            double lon = parseDoubleProperty(request, "lon", 0);
            OverlayNode overlayNode = new OverlayNode((float)lat, (float)lon);
            if (!overlayNode.snap(graph, null))
                throw new ServiceException(ERR_CLOSEST_VERTEX_NOT_FOUND);
            if (isGeoJson) writeOverlayNodeAsFeaturePoint(baos, overlayNode);
            else writeOverlayNodeAsCsv(baos, overlayNode);
            response = baos.toByteArray();
            return new ServiceResponse(HTTP_200, isGeoJson ? MIME_JSON : MIME_TEXT, response);
        }

        return null;
    }

}