/*
 * $Id: StateHandler.java 4566 2017-11-19 16:53:59Z carsten $
 *
 * Copyright 2011 - Carsten Moeller Software
 * All rights reserved.
 * Use is subject to license terms.
 */

package de.cm.osm2po.service.handlers;

import static de.cm.osm2po.service.ServiceHelpers.loadUiState;
import static de.cm.osm2po.service.ServiceHelpers.saveUiState;
import static de.cm.osm2po.service.errors.ErrorEnum.ERR_NO_DATA_FOUND;

import java.io.ByteArrayOutputStream;
import java.util.Properties;

import de.cm.osm2po.Config;
import de.cm.osm2po.routing.Graph;
import de.cm.osm2po.service.Osm2poService;
import de.cm.osm2po.service.ServiceHandler;
import de.cm.osm2po.service.ServiceResourcesChecker;
import de.cm.osm2po.service.ServiceResponse;
import de.cm.osm2po.service.errors.ServiceException;

/**
 * Behandelt Requests des {@link Osm2poService}.
 * @author (c) 2015 - Carsten Moeller - info@osm2po
 */
public class StateHandler implements ServiceHandler {
    @Override
    public ServiceResponse handle(final String cmd, Config config, Graph graph,
            Properties request, ServiceResourcesChecker lttc) throws Exception {

        byte[] response = null;

        if ("st".equals(cmd)) { // Store Tsp State
            ByteArrayOutputStream baos = new ByteArrayOutputStream(0x400);
            String tspStateName = request.getProperty("tspStateName");
            String tspStateKey = tspStateName + "-" + graph.getGraphId();
            String tspStateVal = request.getProperty("tspState", "[]");
            saveUiState(tspStateKey, tspStateVal);
            baos.write(("#'" + tspStateName + "' saved.").getBytes());
            response = baos.toByteArray();

        } else if ("rt".equals(cmd)) { // Reload Tsp State
            ByteArrayOutputStream baos = new ByteArrayOutputStream(0x400);
            String tspStateName = request.getProperty("tspStateName");
            String tspStateKey = tspStateName + "-" + graph.getGraphId();
            String tspStateVal = loadUiState(tspStateKey);
            if (null == tspStateVal) throw new ServiceException(ERR_NO_DATA_FOUND,
                    "'" + tspStateName + "' Graph: " + graph.getGraphId());
            else {
                baos.write(tspStateVal.getBytes());
                response = baos.toByteArray();
            }
        }

        if (null == response) return null;
        else return new ServiceResponse(HTTP_200, MIME_TEXT, response);
    }

}